import re

import dash_bootstrap_components as dbc
import plotly.graph_objs as go
from dash import dash_table, dcc, html

from dashboard.config import fail_emoji, info_emoji, manual_emoji, pass_emoji


def version_tuple(version):
    version = re.sub("[a-zA-Z]", "", version)
    if version == "" or version == "-" or version == " " or version == "_":
        return version
    else:
        if "." in version:
            delimiter = "."
        elif "-" in version:
            delimiter = "-"
        elif "_" in version:
            delimiter = "_"
        else:
            delimiter = None

        # clean up all the strings that end with . or - or _ (few cases)
        while version[-1] == ".":
            version = version.replace(".", "", 1)

        while version[-1] == "-":
            version = version.replace("-", "", 1)

        while version[-1] == "_":
            version = version.replace("_", "", 1)

        if delimiter:
            return tuple(
                int(segment) for segment in version.split(delimiter) if segment
            )
        else:
            return version


def map_status_to_icon(status):
    if status == "FAIL":
        return fail_emoji
    elif status == "PASS":
        return pass_emoji
    elif status == "INFO":
        return info_emoji
    elif status == "MANUAL":
        return manual_emoji
    return status


def get_section_containers_cis(data, section_1, section_2):
    data["STATUS"] = data["STATUS"].apply(map_status_to_icon)
    data.sort_values(by=section_1, key=lambda x: x.map(version_tuple), inplace=True)
    data[section_1] = data[section_1].astype(str)

    findings_counts_section = (
        data.groupby([section_2, "STATUS"]).size().unstack(fill_value=0)
    )
    findings_counts_id = (
        data.groupby([section_1, "STATUS"]).size().unstack(fill_value=0)
    )

    section_containers = []

    for section in data[section_2].unique():
        success_section = (
            findings_counts_section.loc[section, pass_emoji]
            if pass_emoji in findings_counts_section.columns
            else 0
        )
        failed_section = (
            findings_counts_section.loc[section, fail_emoji]
            if fail_emoji in findings_counts_section.columns
            else 0
        )

        fig_section = go.Figure(
            data=[
                go.Bar(
                    name="Failed",
                    x=[failed_section],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#e77676"),
                    width=[0.8],
                ),
                go.Bar(
                    name="Success",
                    x=[success_section],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#45cc6e"),
                    width=[0.8],
                ),
            ]
        )

        fig_section.update_layout(
            barmode="stack",
            margin=dict(l=10, r=10, t=10, b=10),
            paper_bgcolor="rgba(0,0,0,0)",
            plot_bgcolor="rgba(0,0,0,0)",
            showlegend=False,
            width=350,
            height=30,
            xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            annotations=[
                dict(
                    x=success_section + failed_section,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(success_section),
                    showarrow=False,
                    font=dict(color="#45cc6e", size=14),
                    xanchor="left",
                    yanchor="middle",
                ),
                dict(
                    x=0,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(failed_section),
                    showarrow=False,
                    font=dict(color="#e77676", size=14),
                    xanchor="right",
                    yanchor="middle",
                ),
            ],
        )

        fig_section.add_annotation(
            x=50,
            y=0,
            text="",
            showarrow=False,
            align="center",
            xanchor="center",
            yanchor="middle",
            textangle=0,
        )

        fig_section.add_annotation(
            x=failed_section,
            y=0.3,
            text="|",
            showarrow=False,
            align="center",
            xanchor="center",
            yanchor="middle",
            textangle=0,
            font=dict(size=20),
        )

        graph_section = dcc.Graph(
            figure=fig_section, config={"staticPlot": True}, className="info-bar"
        )

        graph_div = html.Div(graph_section, className="graph-section")

        direct_internal_items = []
        for req_id in data[data[section_2] == section][section_1].unique():
            specific_data = data[
                (data[section_2] == section) & (data[section_1] == req_id)
            ]
            success_req = (
                findings_counts_id.loc[req_id, pass_emoji]
                if pass_emoji in findings_counts_id.columns
                else 0
            )
            failed_req = (
                findings_counts_id.loc[req_id, fail_emoji]
                if fail_emoji in findings_counts_id.columns
                else 0
            )

            # Create the DataTable for req_id
            data_table = dash_table.DataTable(
                data=specific_data.to_dict("records"),
                columns=[
                    {"name": i, "id": i}
                    for i in ["CHECKID", "STATUS", "REGION", "ACCOUNTID", "RESOURCEID"]
                ],
                style_table={"overflowX": "auto"},
                style_as_list_view=True,
                style_cell={"textAlign": "left", "padding": "5px"},
            )

            # Create the graph for req_id
            fig_req = go.Figure(
                data=[
                    go.Bar(
                        name="Failed",
                        x=[failed_req],
                        y=[""],
                        orientation="h",
                        marker=dict(color="#e77676"),
                    ),
                    go.Bar(
                        name="Success",
                        x=[success_req],
                        y=[""],
                        orientation="h",
                        marker=dict(color="#45cc6e"),
                    ),
                ]
            )

            fig_req.update_layout(
                barmode="stack",
                margin=dict(l=10, r=10, t=10, b=10),
                paper_bgcolor="rgba(0,0,0,0)",
                plot_bgcolor="rgba(0,0,0,0)",
                showlegend=False,
                width=350,
                height=30,
                xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                annotations=[
                    dict(
                        x=success_req + failed_req,
                        y=0,
                        xref="x",
                        yref="y",
                        text=str(success_req),
                        showarrow=False,
                        font=dict(color="#45cc6e", size=14),
                        xanchor="left",
                        yanchor="middle",
                    ),
                    dict(
                        x=0,
                        y=0,
                        xref="x",
                        yref="y",
                        text=str(failed_req),
                        showarrow=False,
                        font=dict(color="#e77676", size=14),
                        xanchor="right",
                        yanchor="middle",
                    ),
                ],
            )

            fig_req.add_annotation(
                x=50,
                y=0,
                text="",
                showarrow=False,
                align="center",
                xanchor="center",
                yanchor="middle",
                textangle=0,
            )

            fig_req.add_annotation(
                x=failed_req,
                y=0.3,
                text="|",
                showarrow=False,
                align="center",
                xanchor="center",
                yanchor="middle",
                textangle=0,
                font=dict(size=20),
            )

            graph_req = dcc.Graph(
                figure=fig_req, config={"staticPlot": True}, className="info-bar-child"
            )

            graph_div_req = html.Div(graph_req, className="graph-section-req")

            title_internal = (
                f"{req_id} - {specific_data['REQUIREMENTS_DESCRIPTION'].iloc[0]}"
            )

            # Cut the title if it's too long
            title_internal = (
                title_internal[:130] + " ..."
                if len(title_internal) > 130
                else title_internal
            )

            internal_accordion_item = dbc.AccordionItem(
                title=title_internal,
                children=[html.Div([data_table], className="inner-accordion-content")],
            )

            internal_section_container = html.Div(
                [
                    graph_div_req,
                    dbc.Accordion(
                        [internal_accordion_item], start_collapsed=True, flush=True
                    ),
                ],
                className="accordion-inner--child",
            )

            direct_internal_items.append(internal_section_container)

        accordion_item = dbc.AccordionItem(
            title=f"{section}", children=direct_internal_items
        )
        section_container = html.Div(
            [
                graph_div,
                dbc.Accordion([accordion_item], start_collapsed=True, flush=True),
            ],
            className="accordion-inner",
        )

        section_containers.append(section_container)

    return html.Div(section_containers, className="compliance-data-layout")


def get_section_containers_format1(data, section_1, section_2):
    data["STATUS"] = data["STATUS"].apply(map_status_to_icon)
    data.sort_values(by=section_2, key=lambda x: x.map(version_tuple), inplace=True)
    data[section_2] = data[section_2].astype(str)

    findings_counts_section = (
        data.groupby([section_1, "STATUS"]).size().unstack(fill_value=0)
    )
    findings_counts_id = (
        data.groupby([section_2, "STATUS"]).size().unstack(fill_value=0)
    )

    section_containers = []

    for section in data[section_1].unique():
        success_section = (
            findings_counts_section.loc[section, pass_emoji]
            if pass_emoji in findings_counts_section.columns
            else 0
        )
        failed_section = (
            findings_counts_section.loc[section, fail_emoji]
            if fail_emoji in findings_counts_section.columns
            else 0
        )

        fig_section = go.Figure(
            data=[
                go.Bar(
                    name="Failed",
                    x=[failed_section],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#e77676"),
                    width=[0.8],
                ),
                go.Bar(
                    name="Success",
                    x=[success_section],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#45cc6e"),
                    width=[0.8],
                ),
            ]
        )

        fig_section.update_layout(
            barmode="stack",
            margin=dict(l=10, r=10, t=10, b=10),
            paper_bgcolor="rgba(0,0,0,0)",
            plot_bgcolor="rgba(0,0,0,0)",
            showlegend=False,
            width=350,
            height=30,
            xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            annotations=[
                dict(
                    x=success_section + failed_section,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(success_section),
                    showarrow=False,
                    font=dict(color="#45cc6e", size=14),
                    xanchor="left",
                    yanchor="middle",
                ),
                dict(
                    x=0,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(failed_section),
                    showarrow=False,
                    font=dict(color="#e77676", size=14),
                    xanchor="right",
                    yanchor="middle",
                ),
            ],
        )

        fig_section.add_annotation(
            x=50,
            y=0,
            text="",
            showarrow=False,
            align="center",
            xanchor="center",
            yanchor="middle",
            textangle=0,
        )

        fig_section.add_annotation(
            x=failed_section,
            y=0.3,
            text="|",
            showarrow=False,
            align="center",
            xanchor="center",
            yanchor="middle",
            textangle=0,
            font=dict(size=20),
        )

        graph_section = dcc.Graph(
            figure=fig_section, config={"staticPlot": True}, className="info-bar"
        )

        graph_div = html.Div(graph_section, className="graph-section")

        direct_internal_items = []
        for req_id in data[data[section_1] == section][section_2].unique():
            specific_data = data[
                (data[section_1] == section) & (data[section_2] == req_id)
            ]
            success_req = (
                findings_counts_id.loc[req_id, pass_emoji]
                if pass_emoji in findings_counts_id.columns
                else 0
            )
            failed_req = (
                findings_counts_id.loc[req_id, fail_emoji]
                if fail_emoji in findings_counts_id.columns
                else 0
            )

            # Create the DataTable for req_id
            data_table = dash_table.DataTable(
                data=specific_data.to_dict("records"),
                columns=[
                    {"name": i, "id": i}
                    for i in ["CHECKID", "STATUS", "REGION", "ACCOUNTID", "RESOURCEID"]
                ],
                style_table={"overflowX": "auto"},
                style_as_list_view=True,
                style_cell={"textAlign": "left", "padding": "5px"},
            )

            # Create the graph for req_id
            fig_req = go.Figure(
                data=[
                    go.Bar(
                        name="Failed",
                        x=[failed_req],
                        y=[""],
                        orientation="h",
                        marker=dict(color="#e77676"),
                    ),
                    go.Bar(
                        name="Success",
                        x=[success_req],
                        y=[""],
                        orientation="h",
                        marker=dict(color="#45cc6e"),
                    ),
                ]
            )

            fig_req.update_layout(
                barmode="stack",
                margin=dict(l=10, r=10, t=10, b=10),
                paper_bgcolor="rgba(0,0,0,0)",
                plot_bgcolor="rgba(0,0,0,0)",
                showlegend=False,
                width=350,
                height=30,
                xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                annotations=[
                    dict(
                        x=success_req + failed_req,
                        y=0,
                        xref="x",
                        yref="y",
                        text=str(success_req),
                        showarrow=False,
                        font=dict(color="#45cc6e", size=14),
                        xanchor="left",
                        yanchor="middle",
                    ),
                    dict(
                        x=0,
                        y=0,
                        xref="x",
                        yref="y",
                        text=str(failed_req),
                        showarrow=False,
                        font=dict(color="#e77676", size=14),
                        xanchor="right",
                        yanchor="middle",
                    ),
                ],
            )

            fig_req.add_annotation(
                x=50,
                y=0,
                text="",
                showarrow=False,
                align="center",
                xanchor="center",
                yanchor="middle",
                textangle=0,
            )

            fig_req.add_annotation(
                x=failed_req,
                y=0.3,
                text="|",
                showarrow=False,
                align="center",
                xanchor="center",
                yanchor="middle",
                textangle=0,
                font=dict(size=20),
            )

            graph_req = dcc.Graph(
                figure=fig_req, config={"staticPlot": True}, className="info-bar-child"
            )

            graph_div_req = html.Div(graph_req, className="graph-section-req")

            internal_accordion_item = dbc.AccordionItem(
                title=req_id,
                children=[html.Div([data_table], className="inner-accordion-content")],
            )

            internal_section_container = html.Div(
                [
                    graph_div_req,
                    dbc.Accordion(
                        [internal_accordion_item], start_collapsed=True, flush=True
                    ),
                ],
                className="accordion-inner--child",
            )

            direct_internal_items.append(internal_section_container)

        # Cut the title if it's too long
        tittle_external = section[:70] + " ..." if len(section) > 70 else section

        accordion_item = dbc.AccordionItem(
            title=f"{tittle_external}", children=direct_internal_items
        )
        section_container = html.Div(
            [
                graph_div,
                dbc.Accordion([accordion_item], start_collapsed=True, flush=True),
            ],
            className="accordion-inner",
        )

        section_containers.append(section_container)

    return html.Div(section_containers, className="compliance-data-layout")


def get_section_containers_format2(data, section_1, section_2):
    data["STATUS"] = data["STATUS"].apply(map_status_to_icon)

    findings_counts_section = (
        data.groupby([section_2, "STATUS"]).size().unstack(fill_value=0)
    )
    findings_counts_name = (
        data.groupby([section_1, "STATUS"]).size().unstack(fill_value=0)
    )

    section_containers = []

    for name in data[section_1].unique():
        success_name = (
            findings_counts_name.loc[name, pass_emoji]
            if pass_emoji in findings_counts_name.columns
            else 0
        )
        failed_name = (
            findings_counts_name.loc[name, fail_emoji]
            if fail_emoji in findings_counts_name.columns
            else 0
        )

        fig_name = go.Figure(
            data=[
                go.Bar(
                    name="Failed",
                    x=[failed_name],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#e77676"),
                    width=[0.8],
                ),
                go.Bar(
                    name="Success",
                    x=[success_name],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#45cc6e"),
                    width=[0.8],
                ),
            ]
        )

        fig_name.update_layout(
            barmode="stack",
            margin=dict(l=10, r=10, t=10, b=10),
            paper_bgcolor="rgba(0,0,0,0)",
            plot_bgcolor="rgba(0,0,0,0)",
            showlegend=False,
            width=350,
            height=30,
            xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            annotations=[
                dict(
                    x=success_name + failed_name,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(success_name),
                    showarrow=False,
                    font=dict(color="#45cc6e", size=14),
                    xanchor="left",
                    yanchor="middle",
                ),
                dict(
                    x=0,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(failed_name),
                    showarrow=False,
                    font=dict(color="#e77676", size=14),
                    xanchor="right",
                    yanchor="middle",
                ),
            ],
        )

        fig_name.add_annotation(
            x=50,
            y=0,
            text="",
            showarrow=False,
            align="center",
            xanchor="center",
            yanchor="middle",
            textangle=0,
        )

        fig_name.add_annotation(
            x=failed_name,
            y=0.3,
            text="|",
            showarrow=False,
            align="center",
            xanchor="center",
            yanchor="middle",
            textangle=0,
            font=dict(size=20),
        )

        graph_name = dcc.Graph(
            figure=fig_name, config={"staticPlot": True}, className="info-bar"
        )

        graph_div = html.Div(graph_name, className="graph-section")

        direct_internal_items = []
        for section in data[data[section_1] == name][section_2].unique():
            specific_data = data[
                (data[section_1] == name) & (data[section_2] == section)
            ]
            success_section = (
                findings_counts_section.loc[section, pass_emoji]
                if pass_emoji in findings_counts_section.columns
                else 0
            )
            failed_section = (
                findings_counts_section.loc[section, fail_emoji]
                if fail_emoji in findings_counts_section.columns
                else 0
            )

            # Create the DataTable for req_id
            data_table = dash_table.DataTable(
                data=specific_data.to_dict("records"),
                columns=[
                    {"name": i, "id": i}
                    for i in ["CHECKID", "STATUS", "REGION", "ACCOUNTID", "RESOURCEID"]
                ],
                style_table={"overflowX": "auto"},
                style_as_list_view=True,
                style_cell={"textAlign": "left", "padding": "5px"},
            )
            # Create the graph for req_id
            fig_section = go.Figure(
                data=[
                    go.Bar(
                        name="Failed",
                        x=[failed_section],
                        y=[""],
                        orientation="h",
                        marker=dict(color="#e77676"),
                    ),
                    go.Bar(
                        name="Success",
                        x=[success_section],
                        y=[""],
                        orientation="h",
                        marker=dict(color="#45cc6e"),
                    ),
                ]
            )

            fig_section.update_layout(
                barmode="stack",
                margin=dict(l=10, r=10, t=10, b=10),
                paper_bgcolor="rgba(0,0,0,0)",
                plot_bgcolor="rgba(0,0,0,0)",
                showlegend=False,
                width=350,
                height=30,
                xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                annotations=[
                    dict(
                        x=success_section + failed_section,
                        y=0,
                        xref="x",
                        yref="y",
                        text=str(success_section),
                        showarrow=False,
                        font=dict(color="#45cc6e", size=14),
                        xanchor="left",
                        yanchor="middle",
                    ),
                    dict(
                        x=0,
                        y=0,
                        xref="x",
                        yref="y",
                        text=str(failed_section),
                        showarrow=False,
                        font=dict(color="#e77676", size=14),
                        xanchor="right",
                        yanchor="middle",
                    ),
                ],
            )

            fig_section.add_annotation(
                x=50,
                y=0,
                text="",
                showarrow=False,
                align="center",
                xanchor="center",
                yanchor="middle",
                textangle=0,
            )

            fig_section.add_annotation(
                x=failed_section,
                y=0.3,
                text="|",
                showarrow=False,
                align="center",
                xanchor="center",
                yanchor="middle",
                textangle=0,
                font=dict(size=20),
            )

            graph_section = dcc.Graph(
                figure=fig_section,
                config={"staticPlot": True},
                className="info-bar-child",
            )

            graph_div_section = html.Div(graph_section, className="graph-section-req")

            internal_accordion_item = dbc.AccordionItem(
                title=section,
                children=[html.Div([data_table], className="inner-accordion-content")],
            )

            internal_section_container = html.Div(
                [
                    graph_div_section,
                    dbc.Accordion(
                        [internal_accordion_item], start_collapsed=True, flush=True
                    ),
                ],
                className="accordion-inner--child",
            )

            direct_internal_items.append(internal_section_container)

        accordion_item = dbc.AccordionItem(
            title=f"{name}", children=direct_internal_items
        )
        section_container = html.Div(
            [
                graph_div,
                dbc.Accordion([accordion_item], start_collapsed=True, flush=True),
            ],
            className="accordion-inner",
        )

        section_containers.append(section_container)

    return html.Div(section_containers, className="compliance-data-layout")


def get_section_containers_format3(data, section_1, section_2):
    data["STATUS"] = data["STATUS"].apply(map_status_to_icon)
    data.sort_values(by=section_2, key=lambda x: x.map(version_tuple), inplace=True)
    data[section_2] = data[section_2].astype(str)

    findings_counts_section = (
        data.groupby([section_1, "STATUS"]).size().unstack(fill_value=0)
    )
    findings_counts_id = (
        data.groupby([section_2, "STATUS"]).size().unstack(fill_value=0)
    )

    section_containers = []

    for section in data[section_1].unique():
        success_section = (
            findings_counts_section.loc[section, pass_emoji]
            if pass_emoji in findings_counts_section.columns
            else 0
        )
        failed_section = (
            findings_counts_section.loc[section, fail_emoji]
            if fail_emoji in findings_counts_section.columns
            else 0
        )

        fig_section = go.Figure(
            data=[
                go.Bar(
                    name="Failed",
                    x=[failed_section],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#e77676"),
                    width=[0.8],
                ),
                go.Bar(
                    name="Success",
                    x=[success_section],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#45cc6e"),
                    width=[0.8],
                ),
            ]
        )

        fig_section.update_layout(
            barmode="stack",
            margin=dict(l=10, r=10, t=10, b=10),
            paper_bgcolor="rgba(0,0,0,0)",
            plot_bgcolor="rgba(0,0,0,0)",
            showlegend=False,
            width=350,
            height=30,
            xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            annotations=[
                dict(
                    x=success_section + failed_section,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(success_section),
                    showarrow=False,
                    font=dict(color="#45cc6e", size=14),
                    xanchor="left",
                    yanchor="middle",
                ),
                dict(
                    x=0,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(failed_section),
                    showarrow=False,
                    font=dict(color="#e77676", size=14),
                    xanchor="right",
                    yanchor="middle",
                ),
            ],
        )

        fig_section.add_annotation(
            x=50,
            y=0,
            text="",
            showarrow=False,
            align="center",
            xanchor="center",
            yanchor="middle",
            textangle=0,
        )

        fig_section.add_annotation(
            x=failed_section,
            y=0.3,
            text="|",
            showarrow=False,
            align="center",
            xanchor="center",
            yanchor="middle",
            textangle=0,
            font=dict(size=20),
        )

        graph_section = dcc.Graph(
            figure=fig_section, config={"staticPlot": True}, className="info-bar"
        )

        graph_div = html.Div(graph_section, className="graph-section")

        direct_internal_items = []
        for req_id in data[data[section_1] == section][section_2].unique():
            specific_data = data[
                (data[section_1] == section) & (data[section_2] == req_id)
            ]
            success_req = (
                findings_counts_id.loc[req_id, pass_emoji]
                if pass_emoji in findings_counts_id.columns
                else 0
            )
            failed_req = (
                findings_counts_id.loc[req_id, fail_emoji]
                if fail_emoji in findings_counts_id.columns
                else 0
            )

            # Create the DataTable for req_id
            data_table = dash_table.DataTable(
                data=specific_data.to_dict("records"),
                columns=[
                    {"name": i, "id": i}
                    for i in ["CHECKID", "STATUS", "REGION", "ACCOUNTID", "RESOURCEID"]
                ],
                style_table={"overflowX": "auto"},
                style_as_list_view=True,
                style_cell={"textAlign": "left", "padding": "5px"},
            )

            # Create the graph for req_id
            fig_req = go.Figure(
                data=[
                    go.Bar(
                        name="Failed",
                        x=[failed_req],
                        y=[""],
                        orientation="h",
                        marker=dict(color="#e77676"),
                    ),
                    go.Bar(
                        name="Success",
                        x=[success_req],
                        y=[""],
                        orientation="h",
                        marker=dict(color="#45cc6e"),
                    ),
                ]
            )

            fig_req.update_layout(
                barmode="stack",
                margin=dict(l=10, r=10, t=10, b=10),
                paper_bgcolor="rgba(0,0,0,0)",
                plot_bgcolor="rgba(0,0,0,0)",
                showlegend=False,
                width=350,
                height=30,
                xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                annotations=[
                    dict(
                        x=success_req + failed_req,
                        y=0,
                        xref="x",
                        yref="y",
                        text=str(success_req),
                        showarrow=False,
                        font=dict(color="#45cc6e", size=14),
                        xanchor="left",
                        yanchor="middle",
                    ),
                    dict(
                        x=0,
                        y=0,
                        xref="x",
                        yref="y",
                        text=str(failed_req),
                        showarrow=False,
                        font=dict(color="#e77676", size=14),
                        xanchor="right",
                        yanchor="middle",
                    ),
                ],
            )

            fig_req.add_annotation(
                x=50,
                y=0,
                text="",
                showarrow=False,
                align="center",
                xanchor="center",
                yanchor="middle",
                textangle=0,
            )

            fig_req.add_annotation(
                x=failed_req,
                y=0.3,
                text="|",
                showarrow=False,
                align="center",
                xanchor="center",
                yanchor="middle",
                textangle=0,
                font=dict(size=20),
            )

            graph_req = dcc.Graph(
                figure=fig_req, config={"staticPlot": True}, className="info-bar-child"
            )

            graph_div_req = html.Div(graph_req, className="graph-section-req")

            title_internal = (
                f"{req_id} - {specific_data['REQUIREMENTS_DESCRIPTION'].iloc[0]}"
            )
            # Cut the title if it's too long
            title_internal = (
                title_internal[:130] + " ..."
                if len(title_internal) > 130
                else title_internal
            )

            internal_accordion_item = dbc.AccordionItem(
                title=title_internal,
                children=[html.Div([data_table], className="inner-accordion-content")],
            )

            internal_section_container = html.Div(
                [
                    graph_div_req,
                    dbc.Accordion(
                        [internal_accordion_item], start_collapsed=True, flush=True
                    ),
                ],
                className="accordion-inner--child",
            )

            direct_internal_items.append(internal_section_container)

        tittle_external = (
            f"{section} - {specific_data['REQUIREMENTS_DESCRIPTION'].iloc[0]}"
        )
        # Cut the title if it's too long
        tittle_external = (
            tittle_external[:70] + " ..."
            if len(tittle_external) > 70
            else tittle_external
        )
        accordion_item = dbc.AccordionItem(
            title=f"{tittle_external}", children=direct_internal_items
        )
        section_container = html.Div(
            [
                graph_div,
                dbc.Accordion([accordion_item], start_collapsed=True, flush=True),
            ],
            className="accordion-inner",
        )

        section_containers.append(section_container)

    return html.Div(section_containers, className="compliance-data-layout")


def get_section_containers_rbi(data, section_1):
    data["STATUS"] = data["STATUS"].apply(map_status_to_icon)
    data[section_1] = data[section_1].astype(str)
    findings_counts_id = (
        data.groupby([section_1, "STATUS"]).size().unstack(fill_value=0)
    )

    direct_internal_items = []
    for req_id in data[section_1].unique():
        specific_data = data[data[section_1] == req_id]
        success_req = (
            findings_counts_id.loc[req_id, pass_emoji]
            if pass_emoji in findings_counts_id.columns
            else 0
        )
        failed_req = (
            findings_counts_id.loc[req_id, fail_emoji]
            if fail_emoji in findings_counts_id.columns
            else 0
        )

        # Create the DataTable for req_id
        data_table = dash_table.DataTable(
            data=specific_data.to_dict("records"),
            columns=[
                {"name": i, "id": i}
                for i in ["CHECKID", "STATUS", "REGION", "ACCOUNTID", "RESOURCEID"]
            ],
            style_table={"overflowX": "auto"},
            style_as_list_view=True,
            style_cell={"textAlign": "left", "padding": "5px"},
        )

        # Create the graph for req_id
        fig_req = go.Figure(
            data=[
                go.Bar(
                    name="Failed",
                    x=[failed_req],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#e77676"),
                ),
                go.Bar(
                    name="Success",
                    x=[success_req],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#45cc6e"),
                ),
            ]
        )

        fig_req.update_layout(
            barmode="stack",
            margin=dict(l=10, r=10, t=10, b=10),
            paper_bgcolor="rgba(0,0,0,0)",
            plot_bgcolor="rgba(0,0,0,0)",
            showlegend=False,
            width=350,
            height=30,
            xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            annotations=[
                dict(
                    x=success_req + failed_req,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(success_req),
                    showarrow=False,
                    font=dict(color="#45cc6e", size=14),
                    xanchor="left",
                    yanchor="middle",
                ),
                dict(
                    x=0,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(failed_req),
                    showarrow=False,
                    font=dict(color="#e77676", size=14),
                    xanchor="right",
                    yanchor="middle",
                ),
            ],
        )

        fig_req.add_annotation(
            x=50,
            y=0,
            text="",
            showarrow=False,
            align="center",
            xanchor="center",
            yanchor="middle",
            textangle=0,
        )

        fig_req.add_annotation(
            x=failed_req,
            y=0.3,
            text="|",
            showarrow=False,
            align="center",
            xanchor="center",
            yanchor="middle",
            textangle=0,
            font=dict(size=20),
        )

        graph_req = dcc.Graph(
            figure=fig_req, config={"staticPlot": True}, className="info-bar-child"
        )

        graph_div_req = html.Div(graph_req, className="graph-section-req")

        title_internal = (
            f"{req_id} - {specific_data['REQUIREMENTS_DESCRIPTION'].iloc[0]}"
        )
        # Cut the title if it's too long
        title_internal = (
            title_internal[:70] + " ..." if len(title_internal) > 70 else title_internal
        )

        internal_accordion_item = dbc.AccordionItem(
            title=title_internal,
            children=[html.Div([data_table], className="inner-accordion-content")],
        )

        internal_section_container = html.Div(
            [
                graph_div_req,
                dbc.Accordion(
                    [internal_accordion_item], start_collapsed=True, flush=True
                ),
            ],
            className="accordion-inner",
        )

        direct_internal_items.append(internal_section_container)

    return html.Div(direct_internal_items, className="compliance-data-layout")


def get_section_container_iso(data, section_1, section_2):
    data["STATUS"] = data["STATUS"].apply(map_status_to_icon)

    data.sort_values(
        by=section_1,
        key=lambda x: x.map(version_tuple),
        inplace=True,
    )
    data.sort_values(
        by=section_2,
        key=lambda x: x.map(version_tuple),
        inplace=True,
    )
    findings_counts_objetive_id = (
        data.groupby([section_2, "STATUS"]).size().unstack(fill_value=0)
    )
    findings_counts_category = (
        data.groupby([section_1, "STATUS"]).size().unstack(fill_value=0)
    )

    section_containers = []

    for category in data[section_1].unique():
        success_category = (
            findings_counts_category.loc[category, pass_emoji]
            if pass_emoji in findings_counts_category.columns
            else 0
        )
        failed_category = (
            findings_counts_category.loc[category, fail_emoji]
            if fail_emoji in findings_counts_category.columns
            else 0
        )

        fig_category = go.Figure(
            data=[
                go.Bar(
                    name="Failed",
                    x=[failed_category],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#e77676"),
                    width=[0.8],
                ),
                go.Bar(
                    name="Success",
                    x=[success_category],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#45cc6e"),
                    width=[0.8],
                ),
            ]
        )

        fig_category.update_layout(
            barmode="stack",
            margin=dict(l=10, r=10, t=10, b=10),
            paper_bgcolor="rgba(0,0,0,0)",
            plot_bgcolor="rgba(0,0,0,0)",
            showlegend=False,
            width=350,
            height=30,
            xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            annotations=[
                dict(
                    x=success_category + failed_category,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(success_category),
                    showarrow=False,
                    font=dict(color="#45cc6e", size=14),
                    xanchor="left",
                    yanchor="middle",
                ),
                dict(
                    x=0,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(failed_category),
                    showarrow=False,
                    font=dict(color="#e77676", size=14),
                    xanchor="right",
                    yanchor="middle",
                ),
            ],
        )

        fig_category.add_annotation(
            x=50,
            y=0,
            text="",
            showarrow=False,
            align="center",
            xanchor="center",
            yanchor="middle",
            textangle=0,
        )

        fig_category.add_annotation(
            x=failed_category,
            y=0.3,
            text="|",
            showarrow=False,
            align="center",
            xanchor="center",
            yanchor="middle",
            textangle=0,
            font=dict(size=20),
        )

        graph_category = dcc.Graph(
            figure=fig_category, config={"staticPlot": True}, className="info-bar"
        )

        graph_div = html.Div(graph_category, className="graph-section")

        direct_internal_items = []
        for objetive_id in data[data[section_1] == category][section_2].unique():
            specific_data = data[
                (data[section_1] == category) & (data[section_2] == objetive_id)
            ]
            success_objetive_id = (
                findings_counts_objetive_id.loc[objetive_id, pass_emoji]
                if pass_emoji in findings_counts_objetive_id.columns
                else 0
            )
            failed_objetive_id = (
                findings_counts_objetive_id.loc[objetive_id, fail_emoji]
                if fail_emoji in findings_counts_objetive_id.columns
                else 0
            )

            # Create the DataTable for req_id
            data_table = dash_table.DataTable(
                data=specific_data.to_dict("records"),
                columns=[
                    {"name": i, "id": i}
                    for i in ["CHECKID", "STATUS", "REGION", "ACCOUNTID", "RESOURCEID"]
                ],
                style_table={"overflowX": "auto"},
                style_as_list_view=True,
                style_cell={"textAlign": "left", "padding": "5px"},
            )
            # Create the graph for req_id
            fig_objetive_id = go.Figure(
                data=[
                    go.Bar(
                        name="Failed",
                        x=[failed_objetive_id],
                        y=[""],
                        orientation="h",
                        marker=dict(color="#e77676"),
                    ),
                    go.Bar(
                        name="Success",
                        x=[success_objetive_id],
                        y=[""],
                        orientation="h",
                        marker=dict(color="#45cc6e"),
                    ),
                ]
            )

            fig_objetive_id.update_layout(
                barmode="stack",
                margin=dict(l=10, r=10, t=10, b=10),
                paper_bgcolor="rgba(0,0,0,0)",
                plot_bgcolor="rgba(0,0,0,0)",
                showlegend=False,
                width=350,
                height=30,
                xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                annotations=[
                    dict(
                        x=success_objetive_id + failed_objetive_id,
                        y=0,
                        xref="x",
                        yref="y",
                        text=str(success_objetive_id),
                        showarrow=False,
                        font=dict(color="#45cc6e", size=14),
                        xanchor="left",
                        yanchor="middle",
                    ),
                    dict(
                        x=0,
                        y=0,
                        xref="x",
                        yref="y",
                        text=str(failed_objetive_id),
                        showarrow=False,
                        font=dict(color="#e77676", size=14),
                        xanchor="right",
                        yanchor="middle",
                    ),
                ],
            )

            fig_objetive_id.add_annotation(
                x=50,
                y=0,
                text="",
                showarrow=False,
                align="center",
                xanchor="center",
                yanchor="middle",
                textangle=0,
            )

            fig_objetive_id.add_annotation(
                x=failed_objetive_id,
                y=0.3,
                text="|",
                showarrow=False,
                align="center",
                xanchor="center",
                yanchor="middle",
                textangle=0,
                font=dict(size=20),
            )

            graph_objetive_id = dcc.Graph(
                figure=fig_objetive_id,
                config={"staticPlot": True},
                className="info-bar-child",
            )

            graph_div_objetive_id = html.Div(
                graph_objetive_id, className="graph-section-req"
            )

            title_internal = f"{objetive_id} - {specific_data['REQUIREMENTS_ATTRIBUTES_OBJETIVE_NAME'].iloc[0]}"
            # Cut the title if it's too long
            title_internal = (
                title_internal[:130] + " ..."
                if len(title_internal) > 130
                else title_internal
            )

            internal_accordion_item = dbc.AccordionItem(
                title=title_internal,
                children=[html.Div([data_table], className="inner-accordion-content")],
            )

            internal_objetive_id_container = html.Div(
                [
                    graph_div_objetive_id,
                    dbc.Accordion(
                        [internal_accordion_item], start_collapsed=True, flush=True
                    ),
                ],
                className="accordion-inner--child",
            )

            direct_internal_items.append(internal_objetive_id_container)

        accordion_item = dbc.AccordionItem(
            title=f"{category}", children=direct_internal_items
        )
        objetive_id_container = html.Div(
            [
                graph_div,
                dbc.Accordion([accordion_item], start_collapsed=True, flush=True),
            ],
            className="accordion-inner",
        )

        section_containers.append(objetive_id_container)

    return html.Div(section_containers, className="compliance-data-layout")


def get_section_containers_format4(data, section_1):

    data["STATUS"] = data["STATUS"].apply(map_status_to_icon)
    data[section_1] = data[section_1].astype(str)

    findings_counts_service = (
        data.groupby([section_1, "STATUS"]).size().unstack(fill_value=0)
    )

    direct_internal_items = []
    for service in data[section_1].unique():
        specific_data = data[data[section_1] == service]
        success_service = (
            findings_counts_service.loc[service, pass_emoji]
            if pass_emoji in findings_counts_service.columns
            else 0
        )
        failed_service = (
            findings_counts_service.loc[service, fail_emoji]
            if fail_emoji in findings_counts_service.columns
            else 0
        )

        # Create the DataTable for service
        data_table = dash_table.DataTable(
            data=specific_data.to_dict("records"),
            columns=[
                {"name": i, "id": i}
                for i in ["CHECKID", "STATUS", "REGION", "ACCOUNTID", "RESOURCEID"]
            ],
            style_table={"overflowX": "auto"},
            style_as_list_view=True,
            style_cell={"textAlign": "left", "padding": "5px"},
        )

        # Create the graph for service
        fig_service = go.Figure(
            data=[
                go.Bar(
                    name="Failed",
                    x=[failed_service],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#e77676"),
                ),
                go.Bar(
                    name="Success",
                    x=[success_service],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#45cc6e"),
                ),
            ]
        )

        fig_service.update_layout(
            barmode="stack",
            margin=dict(l=10, r=10, t=10, b=10),
            paper_bgcolor="rgba(0,0,0,0)",
            plot_bgcolor="rgba(0,0,0,0)",
            showlegend=False,
            width=350,
            height=30,
            xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            annotations=[
                dict(
                    x=success_service + failed_service,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(success_service),
                    showarrow=False,
                    font=dict(color="#45cc6e", size=14),
                    xanchor="left",
                    yanchor="middle",
                ),
                dict(
                    x=0,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(failed_service),
                    showarrow=False,
                    font=dict(color="#e77676", size=14),
                    xanchor="right",
                    yanchor="middle",
                ),
            ],
        )

        fig_service.add_annotation(
            x=50,
            y=0,
            text="",
            showarrow=False,
            align="center",
            xanchor="center",
            yanchor="middle",
            textangle=0,
        )

        fig_service.add_annotation(
            x=failed_service,
            y=0.3,
            text="|",
            showarrow=False,
            align="center",
            xanchor="center",
            yanchor="middle",
            textangle=0,
            font=dict(size=20),
        )

        graph_service = dcc.Graph(
            figure=fig_service, config={"staticPlot": True}, className="info-bar-child"
        )

        graph_div_service = html.Div(graph_service, className="graph-section-req")
        if "REQUIREMENTS_NAME" not in specific_data.columns:
            title_internal = f"{service}"
        else:
            title_internal = f"{service} - {specific_data['REQUIREMENTS_NAME'].iloc[0]}"

        internal_accordion_item = dbc.AccordionItem(
            title=title_internal,
            children=[html.Div([data_table], className="inner-accordion-content")],
        )

        internal_section_container = html.Div(
            [
                graph_div_service,
                dbc.Accordion(
                    [internal_accordion_item], start_collapsed=True, flush=True
                ),
            ],
            className="accordion-inner",
        )

        direct_internal_items.append(internal_section_container)

    return html.Div(direct_internal_items, className="compliance-data-layout")


def get_section_containers_ens(data, section_1, section_2, section_3, section_4):
    data["STATUS"] = data["STATUS"].apply(map_status_to_icon)

    findings_counts_marco = (
        data.groupby([section_1, "STATUS"]).size().unstack(fill_value=0)
    )

    section_containers = []

    for marco in data[section_1].unique():
        success_marco = (
            findings_counts_marco.loc[marco, pass_emoji]
            if pass_emoji in findings_counts_marco.columns
            else 0
        )
        failed_marco = (
            findings_counts_marco.loc[marco, fail_emoji]
            if fail_emoji in findings_counts_marco.columns
            else 0
        )

        fig_name = go.Figure(
            data=[
                go.Bar(
                    name="Failed",
                    x=[failed_marco],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#e77676"),
                    width=[0.8],
                ),
                go.Bar(
                    name="Success",
                    x=[success_marco],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#45cc6e"),
                    width=[0.8],
                ),
            ]
        )

        fig_name.update_layout(
            barmode="stack",
            margin=dict(l=10, r=10, t=10, b=10),
            paper_bgcolor="rgba(0,0,0,0)",
            plot_bgcolor="rgba(0,0,0,0)",
            showlegend=False,
            width=350,
            height=30,
            xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            annotations=[
                dict(
                    x=success_marco + failed_marco,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(success_marco),
                    showarrow=False,
                    font=dict(color="#45cc6e", size=14),
                    xanchor="left",
                    yanchor="middle",
                ),
                dict(
                    x=0,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(failed_marco),
                    showarrow=False,
                    font=dict(color="#e77676", size=14),
                    xanchor="right",
                    yanchor="middle",
                ),
            ],
        )

        fig_name.add_annotation(
            x=50,
            y=0,
            text="",
            showarrow=False,
            align="center",
            xanchor="center",
            yanchor="middle",
            textangle=0,
        )

        fig_name.add_annotation(
            x=failed_marco,
            y=0.3,
            text="|",
            showarrow=False,
            align="center",
            xanchor="center",
            yanchor="middle",
            textangle=0,
            font=dict(size=20),
        )

        graph_name = dcc.Graph(
            figure=fig_name, config={"staticPlot": True}, className="info-bar"
        )

        graph_div = html.Div(graph_name, className="graph-section")

        direct_internal_items = []

        for categoria in data[data[section_1] == marco][section_2].unique():
            specific_data = data[
                (data[section_1] == marco) & (data[section_2] == categoria)
            ]

            findings_counts_categoria = (
                specific_data.groupby([section_2, "STATUS"])
                .size()
                .unstack(fill_value=0)
            )

            success_categoria = (
                findings_counts_categoria.loc[categoria, pass_emoji]
                if pass_emoji in findings_counts_categoria.columns
                else 0
            )
            failed_categoria = (
                findings_counts_categoria.loc[categoria, fail_emoji]
                if fail_emoji in findings_counts_categoria.columns
                else 0
            )

            # Create the graph for req_id
            fig_section = go.Figure(
                data=[
                    go.Bar(
                        name="Failed",
                        x=[failed_categoria],
                        y=[""],
                        orientation="h",
                        marker=dict(color="#e77676"),
                    ),
                    go.Bar(
                        name="Success",
                        x=[success_categoria],
                        y=[""],
                        orientation="h",
                        marker=dict(color="#45cc6e"),
                    ),
                ]
            )

            fig_section.update_layout(
                barmode="stack",
                margin=dict(l=10, r=10, t=10, b=10),
                paper_bgcolor="rgba(0,0,0,0)",
                plot_bgcolor="rgba(0,0,0,0)",
                showlegend=False,
                width=350,
                height=30,
                xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                annotations=[
                    dict(
                        x=success_categoria + failed_categoria,
                        y=0,
                        xref="x",
                        yref="y",
                        text=str(success_categoria),
                        showarrow=False,
                        font=dict(color="#45cc6e", size=14),
                        xanchor="left",
                        yanchor="middle",
                    ),
                    dict(
                        x=0,
                        y=0,
                        xref="x",
                        yref="y",
                        text=str(failed_categoria),
                        showarrow=False,
                        font=dict(color="#e77676", size=14),
                        xanchor="right",
                        yanchor="middle",
                    ),
                ],
            )

            fig_section.add_annotation(
                x=50,
                y=0,
                text="",
                showarrow=False,
                align="center",
                xanchor="center",
                yanchor="middle",
                textangle=0,
            )

            fig_section.add_annotation(
                x=failed_categoria,
                y=0.3,
                text="|",
                showarrow=False,
                align="center",
                xanchor="center",
                yanchor="middle",
                textangle=0,
                font=dict(size=20),
            )

            graph_section = dcc.Graph(
                figure=fig_section,
                config={"staticPlot": True},
                className="info-bar-child",
            )

            graph_div_section = html.Div(graph_section, className="graph-section-req")

            direct_internal_items_idgrupocontrol = []

            for idgrupocontrol in specific_data[
                (specific_data[section_1] == marco)
                & (specific_data[section_2] == categoria)
            ][section_3].unique():
                specific_data2 = specific_data[
                    (specific_data[section_1] == marco)
                    & (specific_data[section_2] == categoria)
                    & (specific_data[section_3] == idgrupocontrol)
                ]

                findings_counts_idgrupocontrol = (
                    specific_data2.groupby([section_3, "STATUS"])
                    .size()
                    .unstack(fill_value=0)
                )

                success_idgrupocontrol = (
                    findings_counts_idgrupocontrol.loc[idgrupocontrol, pass_emoji]
                    if pass_emoji in findings_counts_idgrupocontrol.columns
                    else 0
                )
                failed_idgrupocontrol = (
                    findings_counts_idgrupocontrol.loc[idgrupocontrol, fail_emoji]
                    if fail_emoji in findings_counts_idgrupocontrol.columns
                    else 0
                )

                # Create the graph for req_id
                fig_idgrupocontrol = go.Figure(
                    data=[
                        go.Bar(
                            name="Failed",
                            x=[failed_idgrupocontrol],
                            y=[""],
                            orientation="h",
                            marker=dict(color="#e77676"),
                        ),
                        go.Bar(
                            name="Success",
                            x=[success_idgrupocontrol],
                            y=[""],
                            orientation="h",
                            marker=dict(color="#45cc6e"),
                        ),
                    ]
                )

                fig_idgrupocontrol.update_layout(
                    barmode="stack",
                    margin=dict(l=10, r=10, t=10, b=10),
                    paper_bgcolor="rgba(0,0,0,0)",
                    plot_bgcolor="rgba(0,0,0,0)",
                    showlegend=False,
                    width=350,
                    height=30,
                    xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                    yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                    annotations=[
                        dict(
                            x=success_idgrupocontrol + failed_idgrupocontrol,
                            y=0,
                            xref="x",
                            yref="y",
                            text=str(success_idgrupocontrol),
                            showarrow=False,
                            font=dict(color="#45cc6e", size=14),
                            xanchor="left",
                            yanchor="middle",
                        ),
                        dict(
                            x=0,
                            y=0,
                            xref="x",
                            yref="y",
                            text=str(failed_idgrupocontrol),
                            showarrow=False,
                            font=dict(color="#e77676", size=14),
                            xanchor="right",
                            yanchor="middle",
                        ),
                    ],
                )

                fig_idgrupocontrol.add_annotation(
                    x=50,
                    y=0,
                    text="",
                    showarrow=False,
                    align="center",
                    xanchor="center",
                    yanchor="middle",
                    textangle=0,
                )

                fig_idgrupocontrol.add_annotation(
                    x=failed_idgrupocontrol,
                    y=0.3,
                    text="|",
                    showarrow=False,
                    align="center",
                    xanchor="center",
                    yanchor="middle",
                    textangle=0,
                    font=dict(size=20),
                )

                graph_idgrupocontrol = dcc.Graph(
                    figure=fig_idgrupocontrol,
                    config={"staticPlot": True},
                    className="info-bar-child",
                )

                graph_div_idgrupocontrol = html.Div(
                    graph_idgrupocontrol, className="graph-section-req"
                )

                direct_internal_items_tipo = []

                for tipo in specific_data2[
                    (specific_data2[section_1] == marco)
                    & (specific_data2[section_2] == categoria)
                    & (specific_data2[section_3] == idgrupocontrol)
                ][section_4].unique():
                    specific_data3 = specific_data2[
                        (specific_data2[section_1] == marco)
                        & (specific_data2[section_2] == categoria)
                        & (specific_data2[section_3] == idgrupocontrol)
                        & (specific_data2[section_4] == tipo)
                    ]

                    findings_counts_tipo = (
                        specific_data3.groupby([section_4, "STATUS"])
                        .size()
                        .unstack(fill_value=0)
                    )

                    success_tipo = (
                        findings_counts_tipo.loc[tipo, pass_emoji]
                        if pass_emoji in findings_counts_tipo.columns
                        else 0
                    )
                    failed_tipo = (
                        findings_counts_tipo.loc[tipo, fail_emoji]
                        if fail_emoji in findings_counts_tipo.columns
                        else 0
                    )

                    # Create the DataTable for each tipo
                    data_table = dash_table.DataTable(
                        data=specific_data3.to_dict("records"),
                        columns=[
                            {"name": i, "id": i}
                            for i in [
                                "CHECKID",
                                "STATUS",
                                "REGION",
                                "ACCOUNTID",
                                "RESOURCEID",
                            ]
                        ],
                        style_table={"overflowX": "auto"},
                        style_as_list_view=True,
                        style_cell={"textAlign": "left", "padding": "5px"},
                    )

                    # Create the graph for req_id
                    fig_tipo = go.Figure(
                        data=[
                            go.Bar(
                                name="Failed",
                                x=[failed_tipo],
                                y=[""],
                                orientation="h",
                                marker=dict(color="#e77676"),
                            ),
                            go.Bar(
                                name="Success",
                                x=[success_tipo],
                                y=[""],
                                orientation="h",
                                marker=dict(color="#45cc6e"),
                            ),
                        ]
                    )

                    fig_tipo.update_layout(
                        barmode="stack",
                        margin=dict(l=10, r=10, t=10, b=10),
                        paper_bgcolor="rgba(0,0,0,0)",
                        plot_bgcolor="rgba(0,0,0,0)",
                        showlegend=False,
                        width=350,
                        height=30,
                        xaxis=dict(
                            showticklabels=False, showgrid=False, zeroline=False
                        ),
                        yaxis=dict(
                            showticklabels=False, showgrid=False, zeroline=False
                        ),
                        annotations=[
                            dict(
                                x=success_tipo + failed_tipo,
                                y=0,
                                xref="x",
                                yref="y",
                                text=str(success_tipo),
                                showarrow=False,
                                font=dict(color="#45cc6e", size=14),
                                xanchor="left",
                                yanchor="middle",
                            ),
                            dict(
                                x=0,
                                y=0,
                                xref="x",
                                yref="y",
                                text=str(failed_tipo),
                                showarrow=False,
                                font=dict(color="#e77676", size=14),
                                xanchor="right",
                                yanchor="middle",
                            ),
                        ],
                    )

                    fig_tipo.add_annotation(
                        x=50,
                        y=0,
                        text="",
                        showarrow=False,
                        align="center",
                        xanchor="center",
                        yanchor="middle",
                        textangle=0,
                    )

                    fig_tipo.add_annotation(
                        x=failed_tipo,
                        y=0.3,
                        text="|",
                        showarrow=False,
                        align="center",
                        xanchor="center",
                        yanchor="middle",
                        textangle=0,
                        font=dict(size=20),
                    )

                    graph_tipo = dcc.Graph(
                        figure=fig_tipo,
                        config={"staticPlot": True},
                        className="info-bar-child",
                    )

                    graph_div_tipo = html.Div(graph_tipo, className="graph-section-req")
                    internal_accordion_item_3 = dbc.AccordionItem(
                        title=tipo,
                        children=[
                            html.Div([data_table], className="inner-accordion-content")
                        ],
                    )
                    internal_section_container_3 = html.Div(
                        [
                            graph_div_tipo,
                            dbc.Accordion(
                                [internal_accordion_item_3],
                                start_collapsed=True,
                                flush=True,
                            ),
                        ],
                        className="accordion-inner--child",
                    )
                    direct_internal_items_tipo.append(internal_section_container_3)

                internal_accordion_item_2 = dbc.AccordionItem(
                    title=idgrupocontrol,
                    children=direct_internal_items_tipo,
                )
                internal_section_container_2 = html.Div(
                    [
                        graph_div_idgrupocontrol,
                        dbc.Accordion(
                            [internal_accordion_item_2],
                            start_collapsed=True,
                            flush=True,
                        ),
                    ],
                    className="accordion-inner--child",
                )
                direct_internal_items_idgrupocontrol.append(
                    internal_section_container_2
                )

            internal_accordion_item = dbc.AccordionItem(
                title=categoria,
                children=direct_internal_items_idgrupocontrol,
            )

            internal_section_container = html.Div(
                [
                    graph_div_section,
                    dbc.Accordion(
                        [internal_accordion_item], start_collapsed=True, flush=True
                    ),
                ],
                className="accordion-inner--child",
            )

            direct_internal_items.append(internal_section_container)

        accordion_item = dbc.AccordionItem(
            title=f"{marco}", children=direct_internal_items
        )
        section_container = html.Div(
            [
                graph_div,
                dbc.Accordion([accordion_item], start_collapsed=True, flush=True),
            ],
            className="accordion-inner",
        )

        section_containers.append(section_container)

    return html.Div(section_containers, className="compliance-data-layout")


def get_section_containers_3_levels(data, section_1, section_2, section_3):
    data["STATUS"] = data["STATUS"].apply(map_status_to_icon)
    findings_counts_marco = (
        data.groupby([section_1, "STATUS"]).size().unstack(fill_value=0)
    )
    section_containers = []
    data[section_1] = data[section_1].astype(str)
    data[section_2] = data[section_2].astype(str)
    data[section_3] = data[section_3].astype(str)

    data.sort_values(
        by=section_3,
        key=lambda x: x.map(extract_numeric_values),
        ascending=True,
        inplace=True,
    )

    for marco in data[section_1].unique():
        success_marco = findings_counts_marco.loc[marco].get(pass_emoji, 0)
        failed_marco = findings_counts_marco.loc[marco].get(fail_emoji, 0)

        fig_name = go.Figure(
            [
                go.Bar(
                    name="Failed",
                    x=[failed_marco],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#e77676"),
                    width=[0.8],
                ),
                go.Bar(
                    name="Success",
                    x=[success_marco],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#45cc6e"),
                    width=[0.8],
                ),
            ]
        )
        fig_name.update_layout(
            barmode="stack",
            margin=dict(l=10, r=10, t=10, b=10),
            paper_bgcolor="rgba(0,0,0,0)",
            plot_bgcolor="rgba(0,0,0,0)",
            showlegend=False,
            width=350,
            height=30,
            xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            annotations=[
                dict(
                    x=success_marco + failed_marco,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(success_marco),
                    showarrow=False,
                    font=dict(color="#45cc6e", size=14),
                    xanchor="left",
                    yanchor="middle",
                ),
                dict(
                    x=0,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(failed_marco),
                    showarrow=False,
                    font=dict(color="#e77676", size=14),
                    xanchor="right",
                    yanchor="middle",
                ),
            ],
        )
        fig_name.add_annotation(
            x=failed_marco,
            y=0.3,
            text="|",
            showarrow=False,
            font=dict(size=20),
            xanchor="center",
            yanchor="middle",
        )

        graph_div = html.Div(
            dcc.Graph(
                figure=fig_name, config={"staticPlot": True}, className="info-bar"
            ),
            className="graph-section",
        )
        direct_internal_items = []

        for categoria in data[data[section_1] == marco][section_2].unique():
            specific_data = data[
                (data[section_1] == marco) & (data[section_2] == categoria)
            ]
            findings_counts_categoria = (
                specific_data.groupby([section_2, "STATUS"])
                .size()
                .unstack(fill_value=0)
            )
            success_categoria = findings_counts_categoria.loc[categoria].get(
                pass_emoji, 0
            )
            failed_categoria = findings_counts_categoria.loc[categoria].get(
                fail_emoji, 0
            )

            fig_section = go.Figure(
                [
                    go.Bar(
                        name="Failed",
                        x=[failed_categoria],
                        y=[""],
                        orientation="h",
                        marker=dict(color="#e77676"),
                        width=[0.8],
                    ),
                    go.Bar(
                        name="Success",
                        x=[success_categoria],
                        y=[""],
                        orientation="h",
                        marker=dict(color="#45cc6e"),
                        width=[0.8],
                    ),
                ]
            )
            fig_section.update_layout(
                barmode="stack",
                margin=dict(l=10, r=10, t=10, b=10),
                paper_bgcolor="rgba(0,0,0,0)",
                plot_bgcolor="rgba(0,0,0,0)",
                showlegend=False,
                width=350,
                height=30,
                xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                annotations=[
                    dict(
                        x=success_categoria + failed_categoria,
                        y=0,
                        xref="x",
                        yref="y",
                        text=str(success_categoria),
                        showarrow=False,
                        font=dict(color="#45cc6e", size=14),
                        xanchor="left",
                        yanchor="middle",
                    ),
                    dict(
                        x=0,
                        y=0,
                        xref="x",
                        yref="y",
                        text=str(failed_categoria),
                        showarrow=False,
                        font=dict(color="#e77676", size=14),
                        xanchor="right",
                        yanchor="middle",
                    ),
                ],
            )
            fig_section.add_annotation(
                x=failed_categoria,
                y=0.3,
                text="|",
                showarrow=False,
                font=dict(size=20),
                xanchor="center",
                yanchor="middle",
            )

            graph_div_section = html.Div(
                dcc.Graph(
                    figure=fig_section,
                    config={"staticPlot": True},
                    className="info-bar-child",
                ),
                className="graph-section-req",
            )
            direct_internal_items_idgrupocontrol = []

            for idgrupocontrol in specific_data[section_3].unique():
                specific_data2 = specific_data[
                    specific_data[section_3] == idgrupocontrol
                ]
                findings_counts_idgrupocontrol = (
                    specific_data2.groupby([section_3, "STATUS"])
                    .size()
                    .unstack(fill_value=0)
                )
                success_idgrupocontrol = findings_counts_idgrupocontrol.loc[
                    idgrupocontrol
                ].get(pass_emoji, 0)
                failed_idgrupocontrol = findings_counts_idgrupocontrol.loc[
                    idgrupocontrol
                ].get(fail_emoji, 0)

                fig_idgrupocontrol = go.Figure(
                    [
                        go.Bar(
                            name="Failed",
                            x=[failed_idgrupocontrol],
                            y=[""],
                            orientation="h",
                            marker=dict(color="#e77676"),
                            width=[0.8],
                        ),
                        go.Bar(
                            name="Success",
                            x=[success_idgrupocontrol],
                            y=[""],
                            orientation="h",
                            marker=dict(color="#45cc6e"),
                            width=[0.8],
                        ),
                    ]
                )
                fig_idgrupocontrol.update_layout(
                    barmode="stack",
                    margin=dict(l=10, r=10, t=10, b=10),
                    paper_bgcolor="rgba(0,0,0,0)",
                    plot_bgcolor="rgba(0,0,0,0)",
                    showlegend=False,
                    width=350,
                    height=30,
                    xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                    yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                    annotations=[
                        dict(
                            x=success_idgrupocontrol + failed_idgrupocontrol,
                            y=0,
                            xref="x",
                            yref="y",
                            text=str(success_idgrupocontrol),
                            showarrow=False,
                            font=dict(color="#45cc6e", size=14),
                            xanchor="left",
                            yanchor="middle",
                        ),
                        dict(
                            x=0,
                            y=0,
                            xref="x",
                            yref="y",
                            text=str(failed_idgrupocontrol),
                            showarrow=False,
                            font=dict(color="#e77676", size=14),
                            xanchor="right",
                            yanchor="middle",
                        ),
                    ],
                )
                fig_idgrupocontrol.add_annotation(
                    x=failed_idgrupocontrol,
                    y=0.3,
                    text="|",
                    showarrow=False,
                    font=dict(size=20),
                    xanchor="center",
                    yanchor="middle",
                )

                graph_div_idgrupocontrol = html.Div(
                    dcc.Graph(
                        figure=fig_idgrupocontrol,
                        config={"staticPlot": True},
                        className="info-bar-child",
                    ),
                    className="graph-section-req",
                )

                data_table = dash_table.DataTable(
                    data=specific_data2.to_dict("records"),
                    columns=[
                        {"name": i, "id": i}
                        for i in [
                            "CHECKID",
                            "STATUS",
                            "REGION",
                            "ACCOUNTID",
                            "RESOURCEID",
                        ]
                    ],
                    style_table={"overflowX": "auto"},
                    style_as_list_view=True,
                    style_cell={"textAlign": "left", "padding": "5px"},
                )

                internal_accordion_item_2 = dbc.AccordionItem(
                    title=idgrupocontrol,
                    children=[
                        graph_div_idgrupocontrol,
                        html.Div([data_table], className="inner-accordion-content"),
                    ],
                )
                direct_internal_items_idgrupocontrol.append(
                    html.Div(
                        [
                            graph_div_idgrupocontrol,
                            dbc.Accordion(
                                [internal_accordion_item_2],
                                start_collapsed=True,
                                flush=True,
                            ),
                        ],
                        className="accordion-inner--child",
                    )
                )

            internal_accordion_item = dbc.AccordionItem(
                title=categoria,
                children=direct_internal_items_idgrupocontrol,
            )
            internal_section_container = html.Div(
                [
                    graph_div_section,
                    dbc.Accordion(
                        [internal_accordion_item], start_collapsed=True, flush=True
                    ),
                ],
                className="accordion-inner--child",
            )
            direct_internal_items.append(internal_section_container)

        accordion_item = dbc.AccordionItem(title=marco, children=direct_internal_items)
        section_container = html.Div(
            [
                graph_div,
                dbc.Accordion([accordion_item], start_collapsed=True, flush=True),
            ],
            className="accordion-inner",
        )
        section_containers.append(section_container)

    return html.Div(section_containers, className="compliance-data-layout")


def get_section_containers_threatscore(data, section_1, section_2, section_3):
    data["STATUS"] = data["STATUS"].apply(map_status_to_icon)
    findings_counts_marco = (
        data.groupby([section_1, "STATUS"]).size().unstack(fill_value=0)
    )
    section_containers = []
    data[section_1] = data[section_1].astype(str)
    data[section_2] = data[section_2].astype(str)
    data[section_3] = data[section_3].astype(str)

    data.sort_values(
        by=section_3,
        key=lambda x: x.map(extract_numeric_values),
        ascending=True,
        inplace=True,
    )

    for marco in data[section_1].unique():
        success_marco = findings_counts_marco.loc[marco].get(pass_emoji, 0)
        failed_marco = findings_counts_marco.loc[marco].get(fail_emoji, 0)

        fig_name = go.Figure(
            [
                go.Bar(
                    name="Failed",
                    x=[failed_marco],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#e77676"),
                    width=[0.8],
                ),
                go.Bar(
                    name="Success",
                    x=[success_marco],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#45cc6e"),
                    width=[0.8],
                ),
            ]
        )
        fig_name.update_layout(
            barmode="stack",
            margin=dict(l=10, r=10, t=10, b=10),
            paper_bgcolor="rgba(0,0,0,0)",
            plot_bgcolor="rgba(0,0,0,0)",
            showlegend=False,
            width=350,
            height=30,
            xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            annotations=[
                dict(
                    x=success_marco + failed_marco,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(success_marco),
                    showarrow=False,
                    font=dict(color="#45cc6e", size=14),
                    xanchor="left",
                    yanchor="middle",
                ),
                dict(
                    x=0,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(failed_marco),
                    showarrow=False,
                    font=dict(color="#e77676", size=14),
                    xanchor="right",
                    yanchor="middle",
                ),
            ],
        )
        fig_name.add_annotation(
            x=failed_marco,
            y=0.3,
            text="|",
            showarrow=False,
            font=dict(size=20),
            xanchor="center",
            yanchor="middle",
        )

        graph_div = html.Div(
            dcc.Graph(
                figure=fig_name, config={"staticPlot": True}, className="info-bar"
            ),
            className="graph-section",
        )
        direct_internal_items = []

        for categoria in data[data[section_1] == marco][section_2].unique():
            specific_data = data[
                (data[section_1] == marco) & (data[section_2] == categoria)
            ]
            findings_counts_categoria = (
                specific_data.groupby([section_2, "STATUS"])
                .size()
                .unstack(fill_value=0)
            )
            success_categoria = findings_counts_categoria.loc[categoria].get(
                pass_emoji, 0
            )
            failed_categoria = findings_counts_categoria.loc[categoria].get(
                fail_emoji, 0
            )

            fig_section = go.Figure(
                [
                    go.Bar(
                        name="Failed",
                        x=[failed_categoria],
                        y=[""],
                        orientation="h",
                        marker=dict(color="#e77676"),
                        width=[0.8],
                    ),
                    go.Bar(
                        name="Success",
                        x=[success_categoria],
                        y=[""],
                        orientation="h",
                        marker=dict(color="#45cc6e"),
                        width=[0.8],
                    ),
                ]
            )
            fig_section.update_layout(
                barmode="stack",
                margin=dict(l=10, r=10, t=10, b=10),
                paper_bgcolor="rgba(0,0,0,0)",
                plot_bgcolor="rgba(0,0,0,0)",
                showlegend=False,
                width=350,
                height=30,
                xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                annotations=[
                    dict(
                        x=success_categoria + failed_categoria,
                        y=0,
                        xref="x",
                        yref="y",
                        text=str(success_categoria),
                        showarrow=False,
                        font=dict(color="#45cc6e", size=14),
                        xanchor="left",
                        yanchor="middle",
                    ),
                    dict(
                        x=0,
                        y=0,
                        xref="x",
                        yref="y",
                        text=str(failed_categoria),
                        showarrow=False,
                        font=dict(color="#e77676", size=14),
                        xanchor="right",
                        yanchor="middle",
                    ),
                ],
            )
            fig_section.add_annotation(
                x=failed_categoria,
                y=0.3,
                text="|",
                showarrow=False,
                font=dict(size=20),
                xanchor="center",
                yanchor="middle",
            )

            graph_div_section = html.Div(
                dcc.Graph(
                    figure=fig_section,
                    config={"staticPlot": True},
                    className="info-bar-child",
                ),
                className="graph-section-req",
            )
            direct_internal_items_idgrupocontrol = []

            for idgrupocontrol in specific_data[section_3].unique():
                specific_data2 = specific_data[
                    specific_data[section_3] == idgrupocontrol
                ]
                findings_counts_idgrupocontrol = (
                    specific_data2.groupby([section_3, "STATUS"])
                    .size()
                    .unstack(fill_value=0)
                )
                success_idgrupocontrol = findings_counts_idgrupocontrol.loc[
                    idgrupocontrol
                ].get(pass_emoji, 0)
                failed_idgrupocontrol = findings_counts_idgrupocontrol.loc[
                    idgrupocontrol
                ].get(fail_emoji, 0)

                fig_idgrupocontrol = go.Figure(
                    [
                        go.Bar(
                            name="Failed",
                            x=[failed_idgrupocontrol],
                            y=[""],
                            orientation="h",
                            marker=dict(color="#e77676"),
                            width=[0.8],
                        ),
                        go.Bar(
                            name="Success",
                            x=[success_idgrupocontrol],
                            y=[""],
                            orientation="h",
                            marker=dict(color="#45cc6e"),
                            width=[0.8],
                        ),
                    ]
                )
                fig_idgrupocontrol.update_layout(
                    barmode="stack",
                    margin=dict(l=10, r=10, t=10, b=10),
                    paper_bgcolor="rgba(0,0,0,0)",
                    plot_bgcolor="rgba(0,0,0,0)",
                    showlegend=False,
                    width=350,
                    height=30,
                    xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                    yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                    annotations=[
                        dict(
                            x=success_idgrupocontrol + failed_idgrupocontrol,
                            y=0,
                            xref="x",
                            yref="y",
                            text=str(success_idgrupocontrol),
                            showarrow=False,
                            font=dict(color="#45cc6e", size=14),
                            xanchor="left",
                            yanchor="middle",
                        ),
                        dict(
                            x=0,
                            y=0,
                            xref="x",
                            yref="y",
                            text=str(failed_idgrupocontrol),
                            showarrow=False,
                            font=dict(color="#e77676", size=14),
                            xanchor="right",
                            yanchor="middle",
                        ),
                    ],
                )
                fig_idgrupocontrol.add_annotation(
                    x=failed_idgrupocontrol,
                    y=0.3,
                    text="|",
                    showarrow=False,
                    font=dict(size=20),
                    xanchor="center",
                    yanchor="middle",
                )

                graph_div_idgrupocontrol = html.Div(
                    dcc.Graph(
                        figure=fig_idgrupocontrol,
                        config={"staticPlot": True},
                        className="info-bar-child",
                    ),
                    className="graph-section-req",
                )

                data_table = dash_table.DataTable(
                    data=specific_data2.to_dict("records"),
                    columns=[
                        {"name": i, "id": i}
                        for i in [
                            "CHECKID",
                            "STATUS",
                            "REGION",
                            "ACCOUNTID",
                            "RESOURCEID",
                        ]
                    ],
                    style_table={"overflowX": "auto"},
                    style_as_list_view=True,
                    style_cell={"textAlign": "left", "padding": "5px"},
                )

                title_internal = f"{idgrupocontrol} - {specific_data2['REQUIREMENTS_DESCRIPTION'].iloc[0]}"

                # Cut the title if it's too long
                title_internal = (
                    title_internal[:130] + " ..."
                    if len(title_internal) > 130
                    else title_internal
                )

                internal_accordion_item_2 = dbc.AccordionItem(
                    title=title_internal,
                    children=[
                        graph_div_idgrupocontrol,
                        html.Div([data_table], className="inner-accordion-content"),
                    ],
                )
                direct_internal_items_idgrupocontrol.append(
                    html.Div(
                        [
                            graph_div_idgrupocontrol,
                            dbc.Accordion(
                                [internal_accordion_item_2],
                                start_collapsed=True,
                                flush=True,
                            ),
                        ],
                        className="accordion-inner--child",
                    )
                )

            internal_accordion_item = dbc.AccordionItem(
                title=categoria,
                children=direct_internal_items_idgrupocontrol,
            )
            internal_section_container = html.Div(
                [
                    graph_div_section,
                    dbc.Accordion(
                        [internal_accordion_item], start_collapsed=True, flush=True
                    ),
                ],
                className="accordion-inner--child",
            )
            direct_internal_items.append(internal_section_container)

        accordion_item = dbc.AccordionItem(title=marco, children=direct_internal_items)
        section_container = html.Div(
            [
                graph_div,
                dbc.Accordion([accordion_item], start_collapsed=True, flush=True),
            ],
            className="accordion-inner",
        )
        section_containers.append(section_container)

    return html.Div(section_containers, className="compliance-data-layout")


# This function extracts and compares up to two numeric values, ensuring correct sorting for version-like strings.
def extract_numeric_values(value):
    numbers = re.findall(r"\d+", str(value))
    if len(numbers) == 3:
        return int(numbers[0]), int(numbers[1]), int(numbers[2])
    elif len(numbers) == 2:
        return int(numbers[0]), int(numbers[1])
    elif len(numbers) == 1:
        return int(numbers[0])
    return 0, 0


def get_section_containers_kisa_ismsp(data, section_1, section_2):
    data["STATUS"] = data["STATUS"].apply(map_status_to_icon)
    data[section_1] = data[section_1].astype(str)
    data[section_2] = data[section_2].astype(str)
    data.sort_values(
        by=section_1,
        key=lambda x: x.map(extract_numeric_values),
        ascending=True,
        inplace=True,
    )

    findings_counts_section = (
        data.groupby([section_2, "STATUS"]).size().unstack(fill_value=0)
    )
    findings_counts_name = (
        data.groupby([section_1, "STATUS"]).size().unstack(fill_value=0)
    )

    section_containers = []

    for name in data[section_1].unique():
        success_name = (
            findings_counts_name.loc[name, pass_emoji]
            if pass_emoji in findings_counts_name.columns
            else 0
        )
        failed_name = (
            findings_counts_name.loc[name, fail_emoji]
            if fail_emoji in findings_counts_name.columns
            else 0
        )

        fig_name = go.Figure(
            data=[
                go.Bar(
                    name="Failed",
                    x=[failed_name],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#e77676"),
                    width=[0.8],
                ),
                go.Bar(
                    name="Success",
                    x=[success_name],
                    y=[""],
                    orientation="h",
                    marker=dict(color="#45cc6e"),
                    width=[0.8],
                ),
            ]
        )

        fig_name.update_layout(
            barmode="stack",
            margin=dict(l=10, r=10, t=10, b=10),
            paper_bgcolor="rgba(0,0,0,0)",
            plot_bgcolor="rgba(0,0,0,0)",
            showlegend=False,
            width=350,
            height=30,
            xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
            annotations=[
                dict(
                    x=success_name + failed_name,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(success_name),
                    showarrow=False,
                    font=dict(color="#45cc6e", size=14),
                    xanchor="left",
                    yanchor="middle",
                ),
                dict(
                    x=0,
                    y=0,
                    xref="x",
                    yref="y",
                    text=str(failed_name),
                    showarrow=False,
                    font=dict(color="#e77676", size=14),
                    xanchor="right",
                    yanchor="middle",
                ),
            ],
        )

        graph_name = dcc.Graph(
            figure=fig_name, config={"staticPlot": True}, className="info-bar"
        )

        graph_div = html.Div(graph_name, className="graph-section")

        direct_internal_items = []

        for section in data[data[section_1] == name][section_2].unique():
            specific_data = data[
                (data[section_1] == name) & (data[section_2] == section)
            ]
            success_section = (
                findings_counts_section.loc[section, pass_emoji]
                if pass_emoji in findings_counts_section.columns
                else 0
            )
            failed_section = (
                findings_counts_section.loc[section, fail_emoji]
                if fail_emoji in findings_counts_section.columns
                else 0
            )

            data_table = dash_table.DataTable(
                data=specific_data.to_dict("records"),
                columns=[
                    {"name": i, "id": i}
                    for i in ["CHECKID", "STATUS", "REGION", "ACCOUNTID", "RESOURCEID"]
                ],
                style_table={"overflowX": "auto"},
                style_as_list_view=True,
                style_cell={"textAlign": "left", "padding": "5px"},
            )

            fig_section = go.Figure(
                data=[
                    go.Bar(
                        name="Failed",
                        x=[failed_section],
                        y=[""],
                        orientation="h",
                        marker=dict(color="#e77676"),
                    ),
                    go.Bar(
                        name="Success",
                        x=[success_section],
                        y=[""],
                        orientation="h",
                        marker=dict(color="#45cc6e"),
                    ),
                ]
            )

            fig_section.update_layout(
                barmode="stack",
                margin=dict(l=10, r=10, t=10, b=10),
                paper_bgcolor="rgba(0,0,0,0)",
                plot_bgcolor="rgba(0,0,0,0)",
                showlegend=False,
                width=350,
                height=30,
                xaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                yaxis=dict(showticklabels=False, showgrid=False, zeroline=False),
                annotations=[
                    dict(
                        x=success_section + failed_section,
                        y=0,
                        xref="x",
                        yref="y",
                        text=str(success_section),
                        showarrow=False,
                        font=dict(color="#45cc6e", size=14),
                        xanchor="left",
                        yanchor="middle",
                    ),
                    dict(
                        x=0,
                        y=0,
                        xref="x",
                        yref="y",
                        text=str(failed_section),
                        showarrow=False,
                        font=dict(color="#e77676", size=14),
                        xanchor="right",
                        yanchor="middle",
                    ),
                ],
            )

            graph_section = dcc.Graph(
                figure=fig_section,
                config={"staticPlot": True},
                className="info-bar-child",
            )

            graph_div_section = html.Div(graph_section, className="graph-section-req")

            internal_accordion_item = dbc.AccordionItem(
                title=section,
                children=[html.Div([data_table], className="inner-accordion-content")],
            )

            internal_section_container = html.Div(
                [
                    graph_div_section,
                    dbc.Accordion(
                        [internal_accordion_item], start_collapsed=True, flush=True
                    ),
                ],
                className="accordion-inner--child",
            )

            direct_internal_items.append(internal_section_container)

        accordion_item = dbc.AccordionItem(
            title=f"{name}", children=direct_internal_items
        )
        section_container = html.Div(
            [
                graph_div,
                dbc.Accordion([accordion_item], start_collapsed=True, flush=True),
            ],
            className="accordion-inner",
        )

        section_containers.append(section_container)

    return html.Div(section_containers, className="compliance-data-layout")



from prowler.lib.check.models import Check, Check_Report_Azure
from prowler.providers.azure.services.entra.entra_client import entra_client


class entra_policy_user_consent_for_verified_apps(Check):
    def execute(self) -> Check_Report_Azure:
        findings = []

        for tenant_domain, auth_policy in entra_client.authorization_policy.items():
            report = Check_Report_Azure(metadata=self.metadata(), resource=auth_policy)
            report.subscription = f"Tenant: {tenant_domain}"
            report.resource_name = getattr(auth_policy, "name", "Authorization Policy")
            report.resource_id = getattr(auth_policy, "id", "authorizationPolicy")
            report.status = "PASS"
            report.status_extended = "Entra does not allow users to consent non-verified apps accessing company data on their behalf."

            if getattr(auth_policy, "default_user_role_permissions", None) and any(
                "ManagePermissionGrantsForSelf.microsoft-user-default-legacy"
                in policy_assigned
                for policy_assigned in getattr(
                    auth_policy.default_user_role_permissions,
                    "permission_grant_policies_assigned",
                    ["ManagePermissionGrantsForSelf.microsoft-user-default-legacy"],
                )
            ):
                report.status = "FAIL"
                report.status_extended = "Entra allows users to consent apps accessing company data on their behalf."

            findings.append(report)

        return findings



from datetime import datetime
from enum import Enum
from typing import Dict, List, Optional

from botocore.exceptions import ClientError
from pydantic.v1 import BaseModel, Field

from prowler.lib.logger import logger
from prowler.lib.scan_filters.scan_filters import is_resource_filtered
from prowler.providers.aws.lib.service.service import AWSService


class StateMachineStatus(str, Enum):
    """Enumeration of possible State Machine statuses."""

    ACTIVE = "ACTIVE"
    DELETING = "DELETING"


class StateMachineType(str, Enum):
    """Enumeration of possible State Machine types."""

    STANDARD = "STANDARD"
    EXPRESS = "EXPRESS"


class LoggingLevel(str, Enum):
    """Enumeration of possible logging levels."""

    ALL = "ALL"
    ERROR = "ERROR"
    FATAL = "FATAL"
    OFF = "OFF"


class EncryptionType(str, Enum):
    """Enumeration of possible encryption types."""

    AWS_OWNED_KEY = "AWS_OWNED_KEY"
    CUSTOMER_MANAGED_KMS_KEY = "CUSTOMER_MANAGED_KMS_KEY"


class CloudWatchLogsLogGroup(BaseModel):
    """
    Represents a CloudWatch Logs Log Group configuration for a State Machine.

    Attributes:
        log_group_arn (str): The ARN of the CloudWatch Logs Log Group.
    """

    log_group_arn: str


class LoggingDestination(BaseModel):
    """
    Represents a logging destination for a State Machine.

    Attributes:
        cloud_watch_logs_log_group (CloudWatchLogsLogGroup): The CloudWatch Logs Log Group configuration.
    """

    cloud_watch_logs_log_group: CloudWatchLogsLogGroup


class LoggingConfiguration(BaseModel):
    """
    Represents the logging configuration for a State Machine.

    Attributes:
        level (LoggingLevel): The logging level.
        include_execution_data (bool): Whether to include execution data in the logs.
        destinations (List[LoggingDestination]): List of logging destinations.
    """

    level: LoggingLevel
    include_execution_data: bool
    destinations: List[LoggingDestination]


class TracingConfiguration(BaseModel):
    """
    Represents the tracing configuration for a State Machine.

    Attributes:
        enabled (bool): Whether X-Ray tracing is enabled.
    """

    enabled: bool


class EncryptionConfiguration(BaseModel):
    """
    Represents the encryption configuration for a State Machine.

    Attributes:
        kms_key_id (Optional[str]): The KMS key ID used for encryption.
        kms_data_key_reuse_period_seconds (Optional[int]): The time in seconds that a KMS data key can be reused.
        type (EncryptionType): The type of encryption used.
    """

    kms_key_id: Optional[str]
    kms_data_key_reuse_period_seconds: Optional[int]
    type: EncryptionType


class StateMachine(BaseModel):
    """
    Represents an AWS Step Functions State Machine.

    Attributes:
        id (str): The unique identifier of the state machine.
        arn (str): The ARN of the state machine.
        name (Optional[str]): The name of the state machine.
        status (StateMachineStatus): The current status of the state machine.
        definition (str): The Amazon States Language definition of the state machine.
        role_arn (str): The ARN of the IAM role used by the state machine.
        type (StateMachineType): The type of the state machine (STANDARD or EXPRESS).
        creation_date (datetime): The creation date and time of the state machine.
        region (str): The region where the state machine is.
        logging_configuration (Optional[LoggingConfiguration]): The logging configuration of the state machine.
        tracing_configuration (Optional[TracingConfiguration]): The tracing configuration of the state machine.
        label (Optional[str]): The label associated with the state machine.
        revision_id (Optional[str]): The revision ID of the state machine.
        description (Optional[str]): A description of the state machine.
        encryption_configuration (Optional[EncryptionConfiguration]): The encryption configuration of the state machine.
        tags (List[Dict]): A list of tags associated with the state machine.
    """

    id: str
    arn: str
    name: Optional[str] = None
    status: StateMachineStatus
    definition: Optional[str] = None
    role_arn: Optional[str] = None
    type: StateMachineType
    creation_date: datetime
    region: str
    logging_configuration: Optional[LoggingConfiguration] = None
    tracing_configuration: Optional[TracingConfiguration] = None
    label: Optional[str] = None
    revision_id: Optional[str] = None
    description: Optional[str] = None
    encryption_configuration: Optional[EncryptionConfiguration] = None
    tags: List[Dict] = Field(default_factory=list)


class StepFunctions(AWSService):
    """
    AWS Step Functions service class to manage state machines.

    This class provides methods to list state machines, describe their details,
    and list their associated tags across different AWS regions.
    """

    def __init__(self, provider):
        """
        Initialize the StepFunctions service.

        Args:
            provider: The AWS provider instance containing regional clients and audit configurations.
        """
        super().__init__(__class__.__name__, provider)
        self.state_machines: Dict[str, StateMachine] = {}
        self.__threading_call__(self._list_state_machines)
        self.__threading_call__(
            self._describe_state_machine, self.state_machines.values()
        )
        self.__threading_call__(
            self._list_state_machine_tags, self.state_machines.values()
        )

    def _list_state_machines(self, regional_client) -> None:
        """
        List AWS Step Functions state machines in the specified region and populate the state_machines dictionary.

        This function retrieves all state machines using pagination, filters them based on audit_resources if provided,
        and creates StateMachine instances to store their basic information.

        Args:
            regional_client: The regional AWS Step Functions client used to interact with the AWS API.
        """
        logger.info("StepFunctions - Listing state machines...")
        try:
            list_state_machines_paginator = regional_client.get_paginator(
                "list_state_machines"
            )

            for page in list_state_machines_paginator.paginate():
                for state_machine_data in page.get("stateMachines", []):
                    try:
                        arn = state_machine_data.get("stateMachineArn")
                        state_machine_id = (
                            arn.split(":")[-1].split("/")[-1] if arn else None
                        )
                        if not self.audit_resources or is_resource_filtered(
                            arn, self.audit_resources
                        ):
                            state_machine = StateMachine(
                                id=state_machine_id,
                                arn=arn,
                                name=state_machine_data.get("name"),
                                type=StateMachineType(
                                    state_machine_data.get("type", "STANDARD")
                                ),
                                creation_date=state_machine_data.get("creationDate"),
                                region=regional_client.region,
                                status=StateMachineStatus.ACTIVE,
                            )

                            self.state_machines[arn] = state_machine
                    except Exception as error:
                        logger.error(
                            f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                        )
        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def _describe_state_machine(self, state_machine: StateMachine) -> None:
        """
        Describe an AWS Step Functions state machine and update its details.

        Args:
            state_machine (StateMachine): The StateMachine instance to describe and update.
        """
        logger.info(
            f"StepFunctions - Describing state machine with ID {state_machine.id} ..."
        )
        try:
            regional_client = self.regional_clients[state_machine.region]
            response = regional_client.describe_state_machine(
                stateMachineArn=state_machine.arn
            )

            state_machine.status = StateMachineStatus(response.get("status"))
            state_machine.definition = response.get("definition")
            state_machine.role_arn = response.get("roleArn")
            state_machine.label = response.get("label")
            state_machine.revision_id = response.get("revisionId")
            state_machine.description = response.get("description")

            logging_config = response.get("loggingConfiguration")
            if logging_config:
                state_machine.logging_configuration = LoggingConfiguration(
                    level=LoggingLevel(logging_config.get("level")),
                    include_execution_data=logging_config.get("includeExecutionData"),
                    destinations=[
                        LoggingDestination(
                            cloud_watch_logs_log_group=CloudWatchLogsLogGroup(
                                log_group_arn=dest["cloudWatchLogsLogGroup"][
                                    "logGroupArn"
                                ]
                            )
                        )
                        for dest in logging_config.get("destinations", [])
                    ],
                )

            tracing_config = response.get("tracingConfiguration")
            if tracing_config:
                state_machine.tracing_configuration = TracingConfiguration(
                    enabled=tracing_config.get("enabled")
                )

            encryption_config = response.get("encryptionConfiguration")
            if encryption_config:
                state_machine.encryption_configuration = EncryptionConfiguration(
                    kms_key_id=encryption_config.get("kmsKeyId"),
                    kms_data_key_reuse_period_seconds=encryption_config.get(
                        "kmsDataKeyReusePeriodSeconds"
                    ),
                    type=EncryptionType(encryption_config.get("type")),
                )

        except ClientError as error:
            if error.response["Error"]["Code"] == "ResourceNotFoundException":
                logger.warning(
                    f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                )
            else:
                logger.error(
                    f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                )
        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def _list_state_machine_tags(self, state_machine: StateMachine) -> None:
        """
        List tags for an AWS Step Functions state machine and update the StateMachine instance.

        Args:
            state_machine (StateMachine): The StateMachine instance to list and update tags for.
        """
        logger.info(
            f"StepFunctions - Listing tags for state machine with ID {state_machine.id} ..."
        )
        try:
            regional_client = self.regional_clients[state_machine.region]

            response = regional_client.list_tags_for_resource(
                resourceArn=state_machine.arn
            )

            state_machine.tags = response.get("tags", [])
        except ClientError as error:
            if error.response["Error"]["Code"] == "ResourceNotFoundException":
                logger.warning(
                    f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                )
            else:
                logger.error(
                    f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                )
        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )



from typing import List

from prowler.lib.check.models import Check, CheckReportM365
from prowler.providers.m365.services.defender.defender_client import defender_client


class defender_malware_policy_notifications_internal_users_malware_enabled(Check):
    """
    Check if notifications for internal users sending malware are enabled in the Defender anti-malware policy.

    Attributes:
        metadata: Metadata associated with the check (inherited from Check).
    """

    def execute(self) -> List[CheckReportM365]:
        """
        Execute the check to verify if notifications for internal users sending malware are enabled.

        This method evaluates the Defender anti-malware policies to ensure internal sender notifications
        are properly configured.

        Returns:
            List[CheckReportM365]: A list of reports with the results of the check.
        """
        findings = []

        if defender_client.malware_policies:
            # Only Default Defender Malware Policy exists
            if not defender_client.malware_rules:
                policy = defender_client.malware_policies[0]

                report = CheckReportM365(
                    metadata=self.metadata(),
                    resource=policy,
                    resource_name=policy.identity,
                    resource_id=policy.identity,
                )

                if self._are_notifications_enabled(policy):
                    # Case 1: Default policy exists and has notifications enabled
                    report.status = "PASS"
                    report.status_extended = f"{policy.identity} is the only policy and notifications for internal users sending malware are enabled."
                else:
                    # Case 5: Default policy exists but doesn't have notifications enabled
                    report.status = "FAIL"
                    report.status_extended = f"{policy.identity} is the only policy and notifications for internal users sending malware are not enabled."
                findings.append(report)

            # Multiple Defender Malware Policies exist
            else:
                default_policy_well_configured = False

                for policy in defender_client.malware_policies:
                    report = CheckReportM365(
                        metadata=self.metadata(),
                        resource=policy,
                        resource_name=policy.identity,
                        resource_id=policy.identity,
                    )

                    if policy.is_default:
                        if not self._are_notifications_enabled(policy):
                            # Case 4: Default policy not configured
                            report.status = "FAIL"
                            report.status_extended = (
                                f"{policy.identity} is the default policy and notifications for internal users sending malware are not enabled, "
                                "but it could be overridden by another well-configured Custom Policy."
                            )
                            findings.append(report)
                        else:
                            # Case 2: Default policy is properly configured
                            report.status = "PASS"
                            report.status_extended = (
                                f"{policy.identity} is the default policy and notifications for internal users sending malware are enabled, "
                                "but it could be overridden by another misconfigured Custom Policy."
                            )
                            default_policy_well_configured = True
                            findings.append(report)
                    else:
                        if not self._are_notifications_enabled(policy):
                            included_resources = []

                            if defender_client.malware_rules[policy.identity].users:
                                included_resources.append(
                                    f"users: {', '.join(defender_client.malware_rules[policy.identity].users)}"
                                )
                            if defender_client.malware_rules[policy.identity].groups:
                                included_resources.append(
                                    f"groups: {', '.join(defender_client.malware_rules[policy.identity].groups)}"
                                )
                            if defender_client.malware_rules[policy.identity].domains:
                                included_resources.append(
                                    f"domains: {', '.join(defender_client.malware_rules[policy.identity].domains)}"
                                )

                            included_resources_str = "; ".join(included_resources)

                            if default_policy_well_configured:
                                # Case 3: Default policy is configured, custom one isn't
                                report.status = "FAIL"
                                report.status_extended = (
                                    f"Custom Malware policy {policy.identity} is not properly configured and includes {included_resources_str}, "
                                    f"with priority {defender_client.malware_rules[policy.identity].priority} (0 is the highest). "
                                    "However, the default policy is properly configured, so entities not included by this custom policy could be correctly protected."
                                )
                                findings.append(report)
                            else:
                                # Case 5: Neither default nor custom policy is properly configured
                                report.status = "FAIL"
                                report.status_extended = (
                                    f"Custom Malware policy {policy.identity} is not properly configured and includes {included_resources_str}, "
                                    f"with priority {defender_client.malware_rules[policy.identity].priority} (0 is the highest). "
                                    "Also, the default policy is not properly configured, so entities not included by this custom policy could not be correctly protected."
                                )
                                findings.append(report)
                        else:
                            included_resources = []

                            if defender_client.malware_rules[policy.identity].users:
                                included_resources.append(
                                    f"users: {', '.join(defender_client.malware_rules[policy.identity].users)}"
                                )
                            if defender_client.malware_rules[policy.identity].groups:
                                included_resources.append(
                                    f"groups: {', '.join(defender_client.malware_rules[policy.identity].groups)}"
                                )
                            if defender_client.malware_rules[policy.identity].domains:
                                included_resources.append(
                                    f"domains: {', '.join(defender_client.malware_rules[policy.identity].domains)}"
                                )

                            included_resources_str = "; ".join(included_resources)

                            if default_policy_well_configured:
                                # Case 2: Both default and custom policies are properly configured
                                report.status = "PASS"
                                report.status_extended = (
                                    f"Custom Malware policy {policy.identity} is properly configured and includes {included_resources_str}, "
                                    f"with priority {defender_client.malware_rules[policy.identity].priority} (0 is the highest). "
                                    "Also, the default policy is properly configured, so entities not included by this custom policy could still be correctly protected."
                                )
                                findings.append(report)
                            else:
                                # Case 6: Default policy not configured, custom policy is
                                report.status = "PASS"
                                report.status_extended = (
                                    f"Custom Malware policy {policy.identity} is properly configured and includes {included_resources_str}, "
                                    f"with priority {defender_client.malware_rules[policy.identity].priority} (0 is the highest). "
                                    "However, the default policy is not properly configured, so entities not included by this custom policy could not be correctly protected."
                                )
                                findings.append(report)

        return findings

    def _are_notifications_enabled(self, policy) -> bool:
        if (
            not policy.is_default
            and policy.identity in defender_client.malware_rules
            and defender_client.malware_rules[policy.identity].state.lower()
            != "enabled"
        ):
            return False

        return (
            policy.enable_internal_sender_admin_notifications
            and policy.internal_sender_admin_address
        )



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.iam.iam_client import iam_client
from prowler.providers.aws.services.iam.lib.policy import (
    check_admin_access,
    check_full_service_access,
)
from prowler.providers.aws.services.iam.lib.privilege_escalation import (
    check_privilege_escalation,
)


class bedrock_api_key_no_administrative_privileges(Check):
    def execute(self):
        findings = []
        for api_key in iam_client.service_specific_credentials:
            if api_key.service_name != "bedrock.amazonaws.com":
                continue
            report = Check_Report_AWS(metadata=self.metadata(), resource=api_key)
            report.status = "PASS"
            report.status_extended = f"API key {api_key.id} in user {api_key.user.name} has no administrative privileges."
            for policy in api_key.user.attached_policies:
                policy_arn = policy["PolicyArn"]
                if policy_arn in iam_client.policies:
                    policy_document = iam_client.policies[policy_arn].document
                    if policy_document:
                        if check_admin_access(policy_document):
                            report.status = "FAIL"
                            report.status_extended = f"API key {api_key.id} in user {api_key.user.name} has administrative privileges through attached policy {policy['PolicyName']}."
                            break
                        elif check_privilege_escalation(policy_document):
                            report.status = "FAIL"
                            report.status_extended = f"API key {api_key.id} in user {api_key.user.name} has privilege escalation through attached policy {policy['PolicyName']}."
                            break
                        elif check_full_service_access("bedrock", policy_document):
                            report.status = "FAIL"
                            report.status_extended = f"API key {api_key.id} in user {api_key.user.name} has full service access through attached policy {policy['PolicyName']}."
                            break
            for inline_policy_name in api_key.user.inline_policies:
                inline_policy_arn = f"{api_key.user.arn}:policy/{inline_policy_name}"
                if inline_policy_arn in iam_client.policies:
                    policy_document = iam_client.policies[inline_policy_arn].document
                    if policy_document:
                        if check_admin_access(policy_document):
                            report.status = "FAIL"
                            report.status_extended = f"API key {api_key.id} in user {api_key.user.name} has administrative privileges through inline policy {inline_policy_name}."
                            break
                        elif check_privilege_escalation(policy_document):
                            report.status = "FAIL"
                            report.status_extended = f"API key {api_key.id} in user {api_key.user.name} has privilege escalation through inline policy {inline_policy_name}."
                            break
                        elif check_full_service_access("bedrock", policy_document):
                            report.status = "FAIL"
                            report.status_extended = f"API key {api_key.id} in user {api_key.user.name} has full service access through inline policy {inline_policy_name}."
                            break
            findings.append(report)

        return findings



from prowler.lib.check.models import Check, Check_Report_Kubernetes
from prowler.providers.kubernetes.services.kubelet.kubelet_client import kubelet_client


class kubelet_disable_anonymous_auth(Check):
    def execute(self) -> Check_Report_Kubernetes:
        findings = []
        for cm in kubelet_client.kubelet_config_maps:
            authentication = cm.kubelet_args.get("authentication", {})
            report = Check_Report_Kubernetes(metadata=self.metadata(), resource=cm)
            report.status = "FAIL"
            report.status_extended = (
                f"Kubelet has anonymous access enabled in config file {cm.name}."
            )
            if not authentication.get("anonymous", {}).get("enabled", False):
                report.status = "PASS"
                report.status_extended = f"Kubelet does not have anonymous access enabled in config file {cm.name}."
            findings.append(report)
        return findings



import base64
import json
import re


def decode_jwt(token: str) -> dict:
    """
    Decodes the payload of a JWT without verifying its signature.

    Args:
        token (str): JWT string in the format 'header.payload.signature'

    Returns:
        dict: A dictionary containing the decoded payload (claims), or an empty dict on failure.
    """
    try:
        # Split the JWT into its 3 parts
        parts = token.split(".")
        if len(parts) != 3:
            raise ValueError(
                "The token does not have the expected three-part structure."
            )

        # Extract and decode the payload (second part)
        payload_b64 = parts[1]

        # Add padding if necessary for base64 decoding
        padding = "=" * (-len(payload_b64) % 4)
        payload_b64 += padding

        payload_bytes = base64.urlsafe_b64decode(payload_b64)
        payload_json = payload_bytes.decode("utf-8")
        payload = json.loads(payload_json)

        return payload

    except Exception as e:
        print(f"Failed to decode the token: {e}")
        return {}


def decode_msal_token(text: str) -> dict:
    """
    Extracts and decodes the payload of a MSAL token from a given string.

    Args:
        text (str): A string that contains the MSAL token, possibly over multiple lines.

    Returns:
        dict: A dictionary containing the decoded payload (claims), or an empty dict on failure.
    """
    try:
        # Join all lines and remove whitespace
        flattened = "".join(text.split())

        # Search for a valid JWT pattern (three base64url parts separated by dots)
        match = re.search(
            r"([A-Za-z0-9-_]+\.[A-Za-z0-9-_]+\.[A-Za-z0-9-_]+)", flattened
        )
        if not match:
            raise ValueError("No valid JWT found in the input.")

        token = match.group(1)
        return decode_jwt(token)

    except Exception as e:
        print(f"Failed to extract and decode the token: {e}")
        return {}



from prowler.providers.aws.services.directconnect.directconnect_service import (
    DirectConnect,
)
from prowler.providers.common.provider import Provider

directconnect_client = DirectConnect(Provider.get_global_provider())



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.elasticache.elasticache_client import (
    elasticache_client,
)


class elasticache_redis_cluster_rest_encryption_enabled(Check):
    def execute(self):
        findings = []
        for repl_group in elasticache_client.replication_groups.values():
            report = Check_Report_AWS(metadata=self.metadata(), resource=repl_group)
            report.status = "FAIL"
            report.status_extended = f"Elasticache Redis cache cluster {repl_group.id} does not have at rest encryption enabled."
            if repl_group.encrypted:
                report.status = "PASS"
                report.status_extended = f"Elasticache Redis cache cluster {repl_group.id} has at rest encryption enabled."

            findings.append(report)

        return findings



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.ecr.ecr_client import ecr_client


class ecr_repositories_scan_vulnerabilities_in_latest_image(Check):
    def execute(self):
        findings = []

        # Get minimun severity to report
        minimum_severity = ecr_client.audit_config.get(
            "ecr_repository_vulnerability_minimum_severity", "MEDIUM"
        )

        for registry in ecr_client.registries.values():
            for repository in registry.repositories:
                # First check if the repository has images
                if len(repository.images_details) > 0:
                    # We only want to check the latest image pushed that is scannable
                    image = repository.images_details[-1]
                    report = Check_Report_AWS(
                        metadata=self.metadata(), resource=repository
                    )
                    report.status = "PASS"
                    status_extended_prefix = f"ECR repository '{repository.name}' has scanned the {image.type} container image with digest '{image.latest_digest}' and tag '{image.latest_tag}' "
                    report.status_extended = (
                        status_extended_prefix + "without findings."
                    )
                    if not image.scan_findings_status:
                        report.status = "FAIL"
                        report.status_extended = (
                            status_extended_prefix + "without a scan."
                        )
                    elif image.scan_findings_status == "FAILED":
                        report.status = "FAIL"
                        report.status_extended = (
                            status_extended_prefix + "with scan status FAILED."
                        )
                    elif (
                        image.scan_findings_status != "FAILED"
                        and image.scan_findings_severity_count
                    ):
                        if (
                            minimum_severity == "CRITICAL"
                            and image.scan_findings_severity_count.critical
                        ):
                            report.status = "FAIL"
                            report.status_extended = (
                                status_extended_prefix
                                + f"with findings: CRITICAL->{image.scan_findings_severity_count.critical}."
                            )
                        elif minimum_severity == "HIGH" and (
                            image.scan_findings_severity_count.critical
                            or image.scan_findings_severity_count.high
                        ):
                            report.status = "FAIL"
                            report.status_extended = (
                                status_extended_prefix
                                + f"with findings: CRITICAL->{image.scan_findings_severity_count.critical}, HIGH->{image.scan_findings_severity_count.high}."
                            )
                        elif minimum_severity == "MEDIUM" and (
                            image.scan_findings_severity_count.critical
                            or image.scan_findings_severity_count.high
                            or image.scan_findings_severity_count.medium
                        ):
                            report.status = "FAIL"
                            report.status_extended = (
                                status_extended_prefix
                                + f"with findings: CRITICAL->{image.scan_findings_severity_count.critical}, HIGH->{image.scan_findings_severity_count.high}, MEDIUM->{image.scan_findings_severity_count.medium}."
                            )

                    findings.append(report)

        return findings



from prowler.lib.check.models import Check, Check_Report_Azure
from prowler.providers.azure.services.app.app_client import app_client


class app_function_ftps_deployment_disabled(Check):
    def execute(self) -> Check_Report_Azure:
        findings = []

        for (
            subscription_name,
            functions,
        ) in app_client.functions.items():
            for function in functions.values():
                report = Check_Report_Azure(metadata=self.metadata(), resource=function)
                report.subscription = subscription_name
                report.status = "FAIL"
                report.status_extended = f"Function {function.name} has {'FTP' if function.ftps_state == 'AllAllowed' else 'FTPS' if function.ftps_state == 'FtpsOnly' else 'FTP or FTPS'} deployment enabled"
                if function.ftps_state == "Disabled":
                    report.status = "PASS"
                    report.status_extended = (
                        f"Function {function.name} has FTP and FTPS deployment disabled"
                    )

                findings.append(report)

        return findings



from prowler.lib.check.models import Check, Check_Report_Azure
from prowler.providers.azure.services.vm.vm_client import vm_client


class vm_scaleset_not_empty(Check):
    """
    Ensure that Azure virtual machine scale sets are not empty (i.e., have no VM instances and no load balancer attached).

    This check evaluates whether each VM scale set has zero VM instances and is not associated with any load balancer backend pool.
    - PASS: The scale set has at least one VM instance or is associated with a load balancer backend pool.
    - FAIL: The scale set has no VM instances and is not associated with any load balancer backend pool (i.e., it is empty).
    """

    def execute(self):
        findings = []
        for subscription, scale_sets in vm_client.vm_scale_sets.items():
            for scale_set in scale_sets.values():
                report = Check_Report_Azure(
                    metadata=self.metadata(), resource=scale_set
                )
                report.subscription = subscription
                if not scale_set.instance_ids:
                    report.status = "FAIL"
                    report.status_extended = f"Scale set '{scale_set.resource_name}' in subscription '{subscription}' is empty: no VM instances present."
                else:
                    report.status = "PASS"
                    report.status_extended = f"Scale set '{scale_set.resource_name}' in subscription '{subscription}' has {len(scale_set.instance_ids)} VM instances."
                findings.append(report)
        return findings



from prowler.lib.check.models import Check, Check_Report_GCP
from prowler.providers.gcp.services.compute.compute_client import compute_client


class compute_subnet_flow_logs_enabled(Check):
    def execute(self) -> Check_Report_GCP:
        findings = []
        for subnet in compute_client.subnets:
            report = Check_Report_GCP(metadata=self.metadata(), resource=subnet)
            report.status = "PASS"
            report.status_extended = f"Subnet {subnet.name} in network {subnet.network} has flow logs enabled."
            if not subnet.flow_logs:
                report.status = "FAIL"
                report.status_extended = f"Subnet {subnet.name} in network {subnet.network} does not have flow logs enabled."
            findings.append(report)

        return findings



from dataclasses import dataclass
from typing import List

from azure.mgmt.authorization import AuthorizationManagementClient

from prowler.lib.logger import logger
from prowler.providers.azure.azure_provider import AzureProvider
from prowler.providers.azure.lib.service.service import AzureService


class IAM(AzureService):
    def __init__(self, provider: AzureProvider):
        super().__init__(AuthorizationManagementClient, provider)
        self.roles, self.custom_roles = self._get_roles()
        self.role_assignments = self._get_role_assignments()

    def _get_roles(self):
        logger.info("IAM - Getting roles...")
        builtin_roles = {}
        custom_roles = {}
        for subscription, client in self.clients.items():
            try:
                builtin_roles.update({subscription: {}})
                custom_roles.update({subscription: {}})
                all_roles = client.role_definitions.list(
                    scope=f"/subscriptions/{self.subscriptions[subscription]}",
                )
                for role in all_roles:
                    if role.role_type == "CustomRole":
                        custom_roles[subscription][role.id] = Role(
                            id=role.id,
                            name=role.role_name,
                            type=role.role_type,
                            assignable_scopes=role.assignable_scopes,
                            permissions=[
                                Permission(
                                    condition=getattr(permission, "condition", ""),
                                    condition_version=getattr(
                                        permission, "condition_version", ""
                                    ),
                                    actions=getattr(permission, "actions", []),
                                )
                                for permission in getattr(role, "permissions", [])
                            ],
                        )
                    else:
                        builtin_roles[subscription][role.id] = Role(
                            id=role.id,
                            name=role.role_name,
                            type=role.role_type,
                            assignable_scopes=role.assignable_scopes,
                            permissions=role.permissions,
                        )
            except Exception as error:
                logger.error(
                    f"Subscription name: {subscription} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                )
        return builtin_roles, custom_roles

    def _get_role_assignments(self):
        logger.info("IAM - Getting role assignments...")
        role_assignments = {}
        for subscription, client in self.clients.items():
            try:
                role_assignments.update({subscription: {}})
                all_role_assignments = client.role_assignments.list_for_subscription(
                    filter="atScope()"
                )
                for role_assignment in all_role_assignments:
                    role_assignments[subscription].update(
                        {
                            role_assignment.id: RoleAssignment(
                                id=role_assignment.id,
                                name=role_assignment.name,
                                scope=role_assignment.scope,
                                agent_id=role_assignment.principal_id,
                                agent_type=role_assignment.principal_type,
                                role_id=role_assignment.role_definition_id.split("/")[
                                    -1
                                ],
                            )
                        }
                    )
            except Exception as error:
                logger.error(
                    f"Subscription name: {subscription} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                )
        return role_assignments


@dataclass
class Permission:
    actions: List[str]
    condition: str
    condition_version: str


@dataclass
class Role:
    id: str
    name: str
    type: str
    assignable_scopes: List[str]
    permissions: List[Permission]


@dataclass
class RoleAssignment:
    """
    Represents an Azure Role Assignment.

    Attributes:
        id: The unique identifier of the role assignment.
        name: The name of the role assignment.
        scope: The scope at which the role assignment applies.
        agent_id: The principal (user, group, service principal, etc.) ID assigned the role.
        agent_type: The type of the principal. Known values: "User", "Group", "ServicePrincipal", "ForeignGroup", and "Device".
        role_id: The ID of the role definition assigned.
    """

    id: str
    name: str
    scope: str
    agent_id: str
    agent_type: str
    role_id: str



from prowler.lib.check.models import Check, Check_Report_GCP
from prowler.providers.gcp.services.compute.compute_client import compute_client


class compute_instance_serial_ports_in_use(Check):
    def execute(self) -> Check_Report_GCP:
        findings = []
        for instance in compute_client.instances:
            report = Check_Report_GCP(metadata=self.metadata(), resource=instance)
            report.status = "PASS"
            report.status_extended = f"VM Instance {instance.name} has Enable Connecting to Serial Ports off."
            if instance.metadata.get("items"):
                for item in instance.metadata["items"]:
                    if item["key"] == "serial-port-enable" and item["value"] in [
                        "1",
                        "true",
                    ]:
                        report.status = "FAIL"
                        report.status_extended = f"VM Instance {instance.name} has Enable Connecting to Serial Ports set to on."
                        break
            findings.append(report)

        return findings



from prowler.lib.check.models import Check, Check_Report_Azure
from prowler.providers.azure.services.storage.storage_client import storage_client


class storage_cross_tenant_replication_disabled(Check):
    """Check if cross-tenant replication is disabled.

    Attributes:
        metadata: Metadata associated with the check (inherited from Check).
    """

    def execute(self) -> Check_Report_Azure:
        """Execute the check for cross-tenant replication.

        This method checks if cross-tenant replication is disabled. If it is, the check passes; otherwise, it fails.

        Returns:
            Check_Report_Azure: A report containing the result of the check.
        """
        findings = []
        for subscription, storage_accounts in storage_client.storage_accounts.items():
            for storage_account in storage_accounts:
                report = Check_Report_Azure(
                    metadata=self.metadata(), resource=storage_account
                )
                report.subscription = subscription
                if not storage_account.allow_cross_tenant_replication:
                    report.status = "PASS"
                    report.status_extended = f"Storage account {storage_account.name} from subscription {subscription} has cross-tenant replication disabled."
                else:
                    report.status = "FAIL"
                    report.status_extended = f"Storage account {storage_account.name} from subscription {subscription} has cross-tenant replication enabled."
                findings.append(report)
        return findings



from prowler.lib.check.models import Check, Check_Report_Azure
from prowler.providers.azure.services.monitor.monitor_client import monitor_client


class monitor_alert_service_health_exists(Check):
    def execute(self) -> list[Check_Report_Azure]:
        findings = []

        for (
            subscription_name,
            activity_log_alerts,
        ) in monitor_client.alert_rules.items():
            for alert_rule in activity_log_alerts:
                # Check if alert rule is enabled and has required Service Health conditions
                if alert_rule.enabled:
                    has_service_health_category = False
                    has_incident_type_incident = False
                    for element in alert_rule.condition.all_of:
                        if (
                            element.field == "category"
                            and element.equals == "ServiceHealth"
                        ):
                            has_service_health_category = True
                        if (
                            element.field == "properties.incidentType"
                            and element.equals == "Incident"
                        ):
                            has_incident_type_incident = True

                    if has_service_health_category and has_incident_type_incident:
                        report = Check_Report_Azure(
                            metadata=self.metadata(), resource=alert_rule
                        )
                        report.subscription = subscription_name
                        report.status = "PASS"
                        report.status_extended = f"There is an activity log alert for Service Health in subscription {subscription_name}."
                        break
            else:
                report = Check_Report_Azure(metadata=self.metadata(), resource={})
                report.subscription = subscription_name
                report.resource_name = "Monitor"
                report.resource_id = "Monitor"
                report.status = "FAIL"
                report.status_extended = f"There is no activity log alert for Service Health in subscription {subscription_name}."

            findings.append(report)

        return findings



import json

from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.lib.utils.utils import detect_secrets_scan
from prowler.providers.aws.services.awslambda.awslambda_client import awslambda_client


class awslambda_function_no_secrets_in_variables(Check):
    def execute(self):
        findings = []
        secrets_ignore_patterns = awslambda_client.audit_config.get(
            "secrets_ignore_patterns", []
        )
        for function in awslambda_client.functions.values():
            report = Check_Report_AWS(metadata=self.metadata(), resource=function)

            report.status = "PASS"
            report.status_extended = (
                f"No secrets found in Lambda function {function.name} variables."
            )

            if function.environment:
                detect_secrets_output = detect_secrets_scan(
                    data=json.dumps(function.environment, indent=2),
                    excluded_secrets=secrets_ignore_patterns,
                    detect_secrets_plugins=awslambda_client.audit_config.get(
                        "detect_secrets_plugins",
                    ),
                )
                original_env_vars = []
                for name, value in function.environment.items():
                    original_env_vars.append(name)
                if detect_secrets_output:
                    secrets_string = ", ".join(
                        [
                            f"{secret['type']} in variable {original_env_vars[secret['line_number'] - 2]}"
                            for secret in detect_secrets_output
                        ]
                    )
                    report.status = "FAIL"
                    report.status_extended = f"Potential secret found in Lambda function {function.name} variables -> {secrets_string}."

            findings.append(report)

        return findings



from typing import Optional

from azure.mgmt.databricks import AzureDatabricksManagementClient
from pydantic import BaseModel

from prowler.lib.logger import logger
from prowler.providers.azure.azure_provider import AzureProvider
from prowler.providers.azure.lib.service.service import AzureService


class Databricks(AzureService):
    """
    Service class for interacting with Azure Databricks workspaces.

    This class initializes the Azure Databricks Management Client for each subscription
    and retrieves all Databricks workspaces within those subscriptions.
    """

    def __init__(self, provider: AzureProvider):
        """
        Initialize the Databricks service with the given Azure provider.

        Args:
            provider: The Azure provider instance containing credentials and configuration.
        """
        super().__init__(AzureDatabricksManagementClient, provider)
        self.workspaces = self._get_workspaces()

    def _get_workspaces(self) -> dict:
        """
        Retrieve all Databricks workspaces for each subscription.

        Returns:
            A dictionary mapping subscription IDs to their Databricks workspaces.
        """
        logger.info("Databricks - Getting workspaces...")
        workspaces = {}
        for subscription, client in self.clients.items():
            try:
                workspaces[subscription] = {}

                for workspace in client.workspaces.list_by_subscription():
                    workspace_parameters = getattr(workspace, "parameters", None)
                    workspace_managed_disk_encryption = getattr(
                        getattr(
                            getattr(workspace, "encryption", None), "entities", None
                        ),
                        "managed_disk",
                        None,
                    )

                    key_vault_properties = getattr(
                        workspace_managed_disk_encryption, "key_vault_properties", None
                    )

                    if key_vault_properties:
                        managed_disk_encryption = ManagedDiskEncryption(
                            key_name=key_vault_properties.key_name,
                            key_version=key_vault_properties.key_version,
                            key_vault_uri=key_vault_properties.key_vault_uri,
                        )
                    else:
                        managed_disk_encryption = None

                    workspaces[subscription][workspace.id] = DatabricksWorkspace(
                        id=workspace.id,
                        name=workspace.name,
                        location=workspace.location,
                        custom_managed_vnet_id=(
                            getattr(
                                workspace_parameters, "custom_virtual_network_id", None
                            ).value
                            if getattr(
                                workspace_parameters, "custom_virtual_network_id", None
                            )
                            else None
                        ),
                        managed_disk_encryption=managed_disk_encryption,
                    )
            except Exception as error:
                logger.error(
                    f"Subscription: {subscription} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                )
        return workspaces


class ManagedDiskEncryption(BaseModel):
    """
    Pydantic model representing the encryption settings for a workspace's managed disks.

    Attributes:
        key_name: The name of the key used for encryption.
        key_version: The version of the key used for encryption.
        key_vault_uri: The URI of the key vault containing the key used for encryption.
    """

    key_name: str
    key_version: str
    key_vault_uri: str


class DatabricksWorkspace(BaseModel):
    """
    Pydantic model representing an Azure Databricks workspace.

    Attributes:
        id: The unique identifier of the workspace.
        name: The name of the workspace.
        location: The Azure region where the workspace is deployed.
        custom_managed_vnet_id: The ID of the custom managed virtual network, if configured.
        managed_disk_encryption: The encryption settings for the workspace's managed disks.
    """

    id: str
    name: str
    location: str
    custom_managed_vnet_id: Optional[str] = None
    managed_disk_encryption: Optional[ManagedDiskEncryption] = None



from typing import List

from prowler.lib.check.models import Check, CheckReportGithub
from prowler.providers.github.services.repository.repository_client import (
    repository_client,
)


class repository_default_branch_deletion_disabled(Check):
    """Check if a repository denies branch deletion

    This class verifies whether each repository denies default branch deletion.
    """

    def execute(self) -> List[CheckReportGithub]:
        """Execute the Github Repository Denies Default Branch Deletion check

        Iterates over all repositories and checks if they deny default branch deletion.

        Returns:
            List[CheckReportGithub]: A list of reports for each repository
        """
        findings = []
        for repo in repository_client.repositories.values():
            if repo.default_branch.branch_deletion is not None:
                report = CheckReportGithub(metadata=self.metadata(), resource=repo)
                report.status = "FAIL"
                report.status_extended = (
                    f"Repository {repo.name} does allow default branch deletion."
                )

                if not repo.default_branch.branch_deletion:
                    report.status = "PASS"
                    report.status_extended = (
                        f"Repository {repo.name} does deny default branch deletion."
                    )

                findings.append(report)

        return findings



from prowler.lib.logger import logger
from prowler.providers.aws.services.cloudtrail.cloudtrail_client import (
    cloudtrail_client,
)


def fixer(region):
    """
    NOTE: Define the S3 bucket name in the fixer_config.yaml file.
    Enable CloudTrail in a region. Requires the cloudtrail:CreateTrail permission.
    Permissions:
    {
        "Version": "2012-10-17",
        "Statement": [
            {
                "Effect": "Allow",
                "Action": "cloudtrail:CreateTrail",
                "Resource": "*"
            }
        ]
    }
    Args:
        region (str): AWS region
    Returns:
        bool: True if CloudTrail is enabled, False otherwise
    """
    try:
        cloudtrail_fixer_config = cloudtrail_client.fixer_config.get(
            "cloudtrail_multi_region_enabled", {}
        )
        regional_client = cloudtrail_client.regional_clients[region]
        args = {
            "Name": cloudtrail_fixer_config.get("TrailName", "DefaultTrail"),
            "S3BucketName": cloudtrail_fixer_config.get("S3BucketName"),
            "IsMultiRegionTrail": cloudtrail_fixer_config.get(
                "IsMultiRegionTrail", True
            ),
            "EnableLogFileValidation": cloudtrail_fixer_config.get(
                "EnableLogFileValidation", True
            ),
        }
        if cloudtrail_fixer_config.get("CloudWatchLogsLogGroupArn"):
            args["CloudWatchLogsLogGroupArn"] = cloudtrail_fixer_config.get(
                "CloudWatchLogsLogGroupArn"
            )
        if cloudtrail_fixer_config.get("CloudWatchLogsRoleArn"):
            args["CloudWatchLogsRoleArn"] = cloudtrail_fixer_config.get(
                "CloudWatchLogsRoleArn"
            )
        if cloudtrail_fixer_config.get("KmsKeyId"):
            args["KmsKeyId"] = cloudtrail_fixer_config.get("KmsKeyId")
        regional_client.create_trail(**args)
    except Exception as error:
        logger.error(
            f"{region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
        )
        return False
    else:
        return True



from prowler.lib.check.models import Check, Check_Report_Kubernetes
from prowler.providers.kubernetes.services.rbac.rbac_client import rbac_client


class rbac_cluster_admin_usage(Check):
    def execute(self) -> Check_Report_Kubernetes:
        findings = []
        # Iterate through the bindings
        for binding in rbac_client.cluster_role_bindings.values():
            # Check if the binding refers to the cluster-admin role
            if binding.roleRef.name == "cluster-admin":
                report = Check_Report_Kubernetes(
                    metadata=self.metadata(), resource=binding.metadata
                )
                report.namespace = (
                    "cluster-wide"
                    if not binding.metadata.namespace
                    else binding.metadata.namespace
                )
                report.status = "MANUAL"
                report.status_extended = f"Cluster Role Binding {binding.metadata.name} uses cluster-admin role."
                findings.append(report)
        return findings



from typing import List

from prowler.lib.check.models import Check, CheckReportM365
from prowler.providers.m365.services.exchange.exchange_client import exchange_client


class exchange_organization_mailtips_enabled(Check):
    """
    Check if MailTips are enabled for Exchange Online.

    Attributes:
        metadata: Metadata associated with the check (inherited from Check).
    """

    def execute(self) -> List[CheckReportM365]:
        """
        Execute the check for MailTips in Exchange Online.

        This method checks if MailTips are enabled in the Exchange organization configuration.

        Returns:
            List[CheckReportM365]: A list of reports containing the result of the check.
        """
        findings = []
        organization_config = exchange_client.organization_config
        if organization_config:
            report = CheckReportM365(
                metadata=self.metadata(),
                resource=organization_config,
                resource_name=organization_config.name,
                resource_id=organization_config.guid,
            )
            report.status = "FAIL"
            report.status_extended = (
                "MailTips are not fully enabled for Exchange Online."
            )

            if (
                organization_config.mailtips_enabled
                and organization_config.mailtips_external_recipient_enabled
                and organization_config.mailtips_group_metrics_enabled
                and organization_config.mailtips_large_audience_threshold
                <= exchange_client.audit_config.get(
                    "recommended_mailtips_large_audience_threshold", 25
                )
            ):
                report.status = "PASS"
                report.status_extended = (
                    "MailTips are fully enabled for Exchange Online."
                )

            findings.append(report)

        return findings



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.s3.s3_client import s3_client


class s3_bucket_event_notifications_enabled(Check):
    """Ensure S3 Buckets have event notifications enabled

    This check will return a FAIL if the S3 Bucket does not have event notifications enabled.
    """

    def execute(self) -> list[Check_Report_AWS]:
        """Execute the s3_bucket_event_notifications_enabled check

        Iterates over all S3 Buckets and checks if they have event notifications enabled.

        Returns:
            list[Check_Report_AWS]: List of Check_Report_AWS objects
        """
        findings = []
        for bucket in s3_client.buckets.values():
            report = Check_Report_AWS(metadata=self.metadata(), resource=bucket)
            report.status = "FAIL"
            report.status_extended = (
                f"S3 Bucket {bucket.name} does not have event notifications enabled."
            )

            if bucket.notification_config:
                report.status = "PASS"
                report.status_extended = (
                    f"S3 Bucket {bucket.name} does have event notifications enabled."
                )

            findings.append(report)

        return findings
